
       //=======================================================//    _\|/_
      //  __  _____           ___                    ___       //      /|\
     //  /      |      ^     |   \  |         ^     |   \     //          _\|/_
    //   \__    |     / \    |___/  |        / \    |___/    //            /|\
   //       \   |    /___\   |  \   |       /___\   |   \   // _\|/_
  //     ___/   |   /     \  |   \  |____  /     \  |___/  //   /|\     
 //                                                       //            _\|/_
//=======================================================//              /|\

/*
 *  scatter.h: definitions for scattering experiments
 *.............................................................................
 *    version 1:  Dec 1993   Piet Hut & Steve McMillan
 *    version 2:
 *.............................................................................
 *     This file includes:
 *  1) definition of state structures
 *  2) declaration of an integrator
 *       ....
 *.............................................................................
 */

#ifndef  STARLAB_SCATTER_H
#  define  STARLAB_SCATTER_H

#include  "sdyn.h"

//-----------------------------------------------------------------------------

// Integration parameters:

#define CHECK_INTERVAL  	20.0
#define DEFAULT_ETA		0.05

// Scattering parameters:

#define LARGE_SEPARATION 10.0      // To be determined (e.g. large mass ratios)
#define LARGE_SEPARATION_FACTOR 10.0 // For all mass ratios
#define TIDAL_TOL_FACTOR 1e-6
#define ENERGY_SAFETY_FACTOR 0.01

#define MIN_INITIAL_SEPARATION 	  10.0
#define MAX_INITIAL_SEPARATION 	  VERY_LARGE_NUMBER

#define ENERGY_TOLERANCE 1e-4        // Maximum absolute energy error allowed
#define MERGER_ENERGY_TOLERANCE 1e-3 // Relax tolerance for *relative* error
				     // in case of merger

// Simple structure representing the dynamical state of a particle:

typedef struct {
    int index;		// > 0 for a real particle
    real mass;		// >= 0 for a real particle
    vector pos;
    vector vel;
} body;

// Helpers:

void print_bodies(ostream&, body*, int prec = 6);

void initialize_bodies(body *);
void set_kepler_from_sdyn(kepler&, sdyn *, sdyn *);

sdyn* mkscat(int, char**);
sdyn* mkscat(char*);

#endif
