
       //=======================================================//    _\|/_
      //  __  _____           ___                    ___       //      /|\
     //  /      |      ^     |   \  |         ^     |   \     //          _\|/_
    //   \__    |     / \    |___/  |        / \    |___/    //            /|\
   //       \   |    /___\   |  \   |       /___\   |   \   // _\|/_
  //     ___/   |   /     \  |   \  |____  /     \  |___/  //   /|\     
 //                                                       //            _\|/_
//=======================================================//              /|\

/*
 *  starbase.h: underlying class for star systems, on the same level as node
 *.............................................................................
 *    version 1:  Apr 1993   Piet Hut & Steve McMillan
 *    version 2:
 *.............................................................................
 *     This file includes:
 *  1) definition of class starbase
 *.............................................................................
 */

#ifndef  STARLAB_STARBASE_H
#  define  STARLAB_STARBASE_H

#include  "starlab_vector.h"
		    	// node.h is commented out deliberately.
			// Because: node.h includes starbase.h.
			// #include  "node.h"
#include  "story.h"
#include  "star/star_support.h"
#include  "star/double_support.h"

class node;		// not included but defined as being some class.
class star;
/*-----------------------------------------------------------------------------
 *  starbase  --  the underlying class for stars
 *-----------------------------------------------------------------------------
 */
class  starbase
    {
    protected:

        node  * the_node;
	story * star_story;

        						
        static real  m_conv_star_to_dyn;   // mass conversion factor
        static real  r_conv_star_to_dyn;   // length conversion factor
        static real  t_conv_star_to_dyn;   // time conversion factor

        static bool  use_hdyn;  

    public:

        starbase(node* n=0);
        starbase(starbase& sb); 

	virtual ~starbase()  {
	  delete star_story;}
	    
        node  * get_node()		       {return the_node;}
	story * get_star_story()               {return star_story;}

        void  set_node(node* n)		       {the_node = n;}
	void  set_star_story(story * ss)       {star_story = ss;}

	virtual ostream & print_star_story(ostream&);
	virtual istream & scan_star_story(istream&, int level = 0);

	bool get_use_hdyn();
	void set_use_hdyn(bool);

	virtual void dump(ostream&, bool);
	
	//              Stellar functions
//              REMOVE for standard STAREV
        virtual stellar_type get_element_type();
        virtual real get_total_mass();
        virtual real get_effective_radius();
        virtual real get_current_time();
        virtual real get_relative_age();
	
        virtual real temperature();
        virtual real get_luminosity();

        virtual vector get_anomal_velocity();
        virtual void set_anomal_velocity(vector);
        virtual void evolve_element(const real);
        virtual real get_evolve_timestep();
        virtual void merge_elements(star*);
        
        virtual real sudden_mass_loss();

        virtual binary_type get_bin_type();
        virtual real get_semi();
        virtual void set_semi(real);
        virtual real get_eccentricity();
        virtual void set_eccentricity(real);

//
//              Scaling:

        void set_stellar_evolution_scaling(real, real, real);
        bool get_stellar_evolution_scaling();
        void print_stellar_evolution_scaling(ostream&);

        real conv_m_star_to_dyn(real);
        real conv_r_star_to_dyn(real);
        real conv_t_star_to_dyn(real);
//
        real conv_m_dyn_to_star(real);
        real conv_r_dyn_to_star(real);
        real conv_t_dyn_to_star(real);
//
	};

typedef  starbase *(*sbpfp)();

inline  starbase * new_starbase()    {return  new starbase;}


#endif
 
