
c
c	Copyright (c) 1986,1987,1988,1989,1990,1991,1992,1993,
c       by Steve McMillan, Drexel University, Philadelphia, PA.
c
c       All rights reserved.
c
c	Redistribution and use in source and binary forms are permitted
c	provided that the above copyright notice and this paragraph are
c	duplicated in all such forms and that any documentation,
c	advertising materials, and other materials related to such
c	distribution and use acknowledge that the software was developed
c	by the author named above.
c
c	THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
c	IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
c	WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
c
c
c---	subroutine:  plt3d
c
c---	3d plot routine, adapted from sas-3 routine of same name
c
c---	calling sequence:
c	call plt3d (a,m,n,x,y,l,alt,az,xlen,xoff,ylen,yoff,zfac,
c    +		zoff,ier)
c
c	a	- real data array, represents height of surface as
c		  function of location in plane
c	m,n	- dimensions of data array a
c	x,y	- real work arrays
c	l	- length of work arrays, .ge. 2*min(m,n)
c	alt, az	- altitude, azimuth viewing angles in degrees
c	xlen,ylen - length of unprojected x,y axes in inches
c	xoff,yoff - offset of origin in inches
c	zfac   - scaling of z-axis from data units to unprojected inches
c	zoff	- offset of z-origin in inches
c	ier	- returns 0 for no error
c
c	Explanatory note (8/17/88, SMcM):
c	--------------------------------
c
c	Altitude is measured from the x-y plane toward the positive z-axis.
c	Azimuth is measured around the z-axis (as seen from above), clockwise
c	from the x-axis.
c
c	The origin (xoff, yoff), in screen coordinates, is the image of the
c	point at the CENTER of the x-y grid and at a height zoff above it.
c
c	Obfuscatory note (12/7/90, SMcM):
c	--------------------------------
c
c	There appears to be a reflection about the line y = x...
c
        subroutine plt3d(a,m,n,x,y,l,alt,az,xlen,xoff,ylen,yoff,
     +                   zfac,zoff,ier)
        save
c
        common /plt3b/ a1,a2,a3,b1,b2,b3,b4
        dimension a(m,n),x(l),y(l)
c
        lmax=2*min0(m,n)
        if(l .lt. lmax) go to 500
c
        taz=az*0.0174532925
        talt=alt*0.0174532925
c
        saz=sin(taz)
        caz=cos(taz)
        sal=sin(talt)
        cal=cos(talt)
c
        xsc=xlen/float(n-1)
        ysc=ylen/float(m-1)
        a1=caz*xsc
        a2=-saz*ysc
        a3=xoff-0.5*(a1*float(n+1)+a2*float(m+1))
        b1=saz*sal*xsc
        b2=caz*sal*ysc
        b3=zfac*cal
        b4=b3*zoff+yoff-0.5*(b1*float(n+1)+b2*float(m+1))
c
        iaz=1
        if(a1 .le. 0.0) iaz=iaz+1
        if(a2 .le. 0.0) iaz=iaz+2
c
        go to (10,20,10,20),iaz
 10     ifirst=1
        istep=1
        ilast=m
        go to 30
 20     ifirst=m
        istep=-1
        ilast=1
c
 30     go to (50,50,40,40),iaz
 40     jfirst=1
        jstep=1
        jlast=n
        go to 60
 50     jfirst=n
        jstep=-1
        jlast=1
c
 60     go to (64,62,62,64),iaz
 62     lli=1
        go to 66
 64     lli=-1
c
 66     ic=0
        ibeg=ifirst+istep
 70     lnth=min0(2*iabs(ibeg-ifirst)+1,lmax)
        if(lli .eq. -1) go to 72
        ll=0
        go to 74
 72     ll=lnth+1
 74     i=ibeg
        j=jfirst
        xx=float(j)
        yy=float(i)
        ll=ll+lli
        x(ll)=a1*xx+a2*yy+a3
        y(ll)=b1*xx+b2*yy+b3*(a(i,j)+zoff)+b4
 80     i=i-istep
        yy=float(i)
        ll=ll+lli
        x(ll)=a1*xx+a2*yy+a3
        y(ll)=b1*xx+b2*yy+b3*(a(i,j)+zoff)+b4
        if(j .eq. jlast) go to 85
        j=j+jstep
        xx=float(j)
        ll=ll+lli
        x(ll)=a1*xx+a2*yy+a3
        y(ll)=b1*xx+b2*yy+b3*(a(i,j)+zoff)+b4
        if(i .ne. ifirst) go to 80
 85     call nxtvu(ic,x,y,lnth,ier)
        if(ier .ne. 0) return
        ic=1
        if(ibeg .eq. ilast) go to 90
        ibeg=ibeg+istep
        go to 70
c
 90     jbeg=jfirst
 100    lnth=min0(2*iabs(jbeg-jlast)+1,lmax)
        if (lli.eq.-1) go to 102
        ll=0
        go to 104
 102    ll=lnth+1
 104    i=ilast
        j=jbeg
        xx=float(j)
        yy=float(i)
        ll=ll+lli
        x(ll)=a1*xx+a2*yy+a3
        y(ll)=b1*xx+b2*yy+b3*(a(i,j)+zoff)+b4
 110    j=j+jstep
        xx=float(j)
        ll=ll+lli
        x(ll)=a1*xx+a2*yy+a3
        y(ll)=b1*xx+b2*yy+b3*(a(i,j)+zoff)+b4
        if(i .eq. ifirst) go to 120
        i=i-istep
        yy=float(i)
        ll=ll+lli
        x(ll)=a1*xx+a2*yy+a3
        y(ll)=b1*xx+b2*yy+b3*(a(i,j)+zoff)+b4
        if(j .ne. jlast) go to 110
 120    call nxtvu(1,x,y,lnth,ier)
        if(ier .ne. 0) return
        jbeg=jbeg+jstep
        if (jbeg.eq.jlast) return
        go to 100
c
 500    ier=2
        end
