
       //=======================================================//    _\|/_
      //  __  _____           ___                    ___       //      /|\
     //  /      |      ^     |   \  |         ^     |   \     //          _\|/_
    //   \__    |     / \    |___/  |        / \    |___/    //            /|\
   //       \   |    /___\   |  \   |       /___\   |   \   // _\|/_
  //     ___/   |   /     \  |   \  |____  /     \  |___/  //   /|\     
 //                                                       //            _\|/_
//=======================================================//              /|\

// Functions associated with log output, snapshots, etc.
//
// Externally visible functions:
//
//	void update_cpu_counters
//	void log_output
//	void snap_output
//	void check_runtime

#include "hdyn.h"
#include "star/dstar_to_kira.h"
#include "kira_local.h"

local void putq(story* s, char* l, int i)		{putiq (s, l, i);}
local void putq(story* s, char* l, unsigned long i)	{putulq(s, l, i);}
local void putq(story* s, char* l, real r)		{putrq (s, l, r);}
local void putq(story* s, char* l, vector v)		{putvq (s, l, v);}

#define PUTLOG(x) putq(b->get_log_story(), #x, b->get_kira_counters()->x);

#define PRLOGC(x) cerr << #x << " = " << b->get_kira_counters()->x << ",  "
#define PRLOGL(x) cerr << #x << " = " << kc->x; \
		  if (kc_prev) cerr << "  (" <<  kc->x - kc_prev->x << ")"; \
		  cerr << endl

void print_counters(kira_counters* kc, kira_counters* kc_prev)
{
    cerr << "\n  Counters: \n";

    cerr << "    CPU time = " << kc->cpu_time;
    if (kc_prev)
	cerr << "  (delta = " << kc->cpu_time - kc_prev->cpu_time << ")";
    cerr << endl;

    PRI(4); PRLOGL(step_top_single);
    PRI(4); PRLOGL(step_top_cm);
    PRI(4); PRLOGL(step_low_level);

    PRI(4); PRLOGL(pert_step);
    PRI(4); PRLOGL(pert_with_list);
    PRI(4); PRLOGL(pert_without_list);
    PRI(4); PRLOGL(perturber_overflow);

    PRI(4); PRLOGL(force_correction);

    PRI(4); PRLOGL(full_unpert_step);
    PRI(4); PRLOGL(full_unpert_orbit);
    PRI(4); PRLOGL(partial_unpert_orbit);

    PRI(4); PRLOGL(tree_change);
    PRI(4); PRLOGL(top_level_combine);
    PRI(4); PRLOGL(low_level_combine);
    PRI(4); PRLOGL(top_level_split);
    PRI(4); PRLOGL(leaf_merge);

    PRI(4); PRLOGL(dm_escapers);
    PRI(4); PRLOGL(dm_massloss);

    PRI(4); PRLOGL(de_total);
    PRI(4); PRLOGL(de_merge);
    PRI(4); PRLOGL(de_massloss);
    PRI(4); PRLOGL(de_kick);

    if (kc_prev)
	*kc_prev = *kc;
}

local void write_counters_to_log(hdyn* b)
{
    // cerr<<"in write_counters_to_log"<<endl;

    PUTLOG(cpu_time);
    PUTLOG(total_cpu_time);

    PUTLOG(step_top_single);
    PUTLOG(step_top_cm);
    PUTLOG(step_low_level);
    PUTLOG(force_correction);

    PUTLOG(pert_step);
    PUTLOG(pert_with_list);
    PUTLOG(pert_without_list);
    PUTLOG(perturber_overflow);

    PUTLOG(full_unpert_step);
    PUTLOG(full_unpert_orbit);
    PUTLOG(partial_unpert_orbit);

    PUTLOG(tree_change);
    PUTLOG(top_level_combine);
    PUTLOG(low_level_combine);
    PUTLOG(top_level_split);

    PUTLOG(leaf_merge);

    PUTLOG(dm_escapers);
    PUTLOG(dm_massloss);

    PUTLOG(initial_etot);
    PUTLOG(de_total);
    PUTLOG(de_merge);
    PUTLOG(de_massloss);
    PUTLOG(de_kick);
}



typedef  struct {
    real  dt;
    hdyn*  b;
} dt_pair, *dt_pair_ptr;

local int compare_dt(const void * pi, const void * pj)	  // increasing dt
{
    if (((dt_pair_ptr) pi)->dt < ((dt_pair_ptr) pj)->dt)
        return -1;
    else if (((dt_pair_ptr)pi)->dt > ((dt_pair_ptr)pj)->dt)
        return +1;
    else
        return 0;
}

typedef  struct {
    real  count;
    hdyn*  b;
} count_pair, *count_pair_ptr;

local int compare_steps(const void * pi, const void * pj) // decreasing count
{
    if (((count_pair_ptr) pi)->count > ((count_pair_ptr) pj)->count)
        return -1;
    else if (((count_pair_ptr)pi)->count < ((count_pair_ptr)pj)->count)
        return +1;
    else
        return 0;
}

#define NTOP 10
#define NBOT 10

local void timestep_stats(hdyn* b)
{
    // Time step statistics.

    real dt_mean = 0, dt_min = VERY_LARGE_NUMBER, dt_max = -dt_min;

    real n_dt = 0;         // actually these are integer counters
    real n_dt_p = 0;
    real total_steps = 0;
    real total_steps_p = 0;

    int count[30], count_p[30];
    int i;
    for (i = 0; i < 30; i++) count[i] = count_p[i] = 0;

    int n = 0;
    for_all_nodes(hdyn, b, bb)  n++;
    n--;

    dt_pair_ptr list_dt = new dt_pair[n];

    count_pair_ptr steps_inst  = new count_pair[n];
    count_pair_ptr steps_delta = new count_pair[n];
    count_pair_ptr f_dir_delta = new count_pair[n];
    count_pair_ptr f_ind_delta = new count_pair[n];

    n = 0;
    int np = 0, nd = 0, ni = 0;

    real delta_steps_tot = 0;
    real delta_f_dir_tot = 0;
    real delta_f_ind_tot = 0;

    for_all_nodes(hdyn, b, bb)
	if (bb != b) {

	    real dt = bb->get_timestep();
	    dt_mean += dt;
	    dt_min = min(dt_min, dt);
	    dt_max = max(dt_max, dt);

	    real steps = rint(1/dt);
	    total_steps += steps;
	    n_dt += 1;

	    i = 0;
	    real dtbin = 1;
	    while (dt < dtbin) {
		i++;
		dtbin *= 0.5;
	    }
	    if (i > 29) i = 29;
	    count[i]++;

	    if (!bb->get_kepler()) {
		total_steps_p += steps;
		n_dt_p += 1;
		count_p[i]++;
	    }

	    if (bb->is_top_level_node()
		|| (bb->get_elder_sister() == NULL && !bb->get_kepler())) {
		list_dt[np].dt = dt;
		list_dt[np].b = bb;
		steps_inst[np].count = steps;
		steps_inst[np].b = bb;
		np++;
	    }

	    steps_delta[n].count = bb->get_steps();
	    if (find_qmatch(bb->get_dyn_story(), "prev_steps"))
		steps_delta[n].count -= getrq(bb->get_dyn_story(),
					      "prev_steps");
	    putrq(bb->get_dyn_story(), "prev_steps", bb->get_steps());
	    steps_delta[n].b = bb;
	    delta_steps_tot += steps_delta[n].count;
	    n++;

	    f_dir_delta[nd].count = bb->get_direct_force();
	    if (find_qmatch(bb->get_dyn_story(), "prev_f_dir"))
		f_dir_delta[nd].count -= getrq(bb->get_dyn_story(),
					       "prev_f_dir");
	    putrq(bb->get_dyn_story(), "prev_f_dir", bb->get_direct_force());
	    f_dir_delta[nd].b = bb;
	    delta_f_dir_tot += f_dir_delta[nd].count;
	    nd++;

	    f_ind_delta[ni].count = bb->get_indirect_force();
	    if (find_qmatch(bb->get_dyn_story(), "prev_f_ind"))
		f_ind_delta[ni].count -= getrq(bb->get_dyn_story(),
					       "prev_f_ind");
	    putrq(bb->get_dyn_story(), "prev_f_ind", bb->get_indirect_force());
	    f_ind_delta[ni].b = bb;
	    delta_f_ind_tot += f_ind_delta[ni].count;
	    ni++;
	}

    cerr << endl << "  Timesteps (mean = " << dt_mean/max((int)rint(n_dt), 1)
	 << "  min = " << dt_min << "  max = " << dt_max << ")" << endl;

    cerr << "  all nodes (bin #1 = 1, dt /= 2):"
	 << endl << "        ";
    for (i =  0; i < 10; i++) fprintf(stderr, " %5d", count[i]);
    cerr << endl << "        ";
    for (i = 10; i < 20; i++) fprintf(stderr, " %5d", count[i]);
    cerr << endl << "        ";
    for (i = 20; i < 30; i++) fprintf(stderr, " %5d", count[i]);

    cerr << endl << endl << "  Weighted timestep sum = "
	 << total_steps/n_dt << endl;

    if (total_steps_p != total_steps) {
	cerr << endl << "  excluding unperturbed systems:"
	     << endl << "        ";
	for (i =  0; i < 10; i++) fprintf(stderr, " %5d", count_p[i]);
	cerr << endl << "        ";
	for (i = 10; i < 20; i++) fprintf(stderr, " %5d", count_p[i]);
	cerr << endl << "        ";
	for (i = 20; i < 30; i++) fprintf(stderr, " %5d", count_p[i]);

	cerr << endl << endl << "  Weighted timestep sum = "
	     << total_steps_p/n_dt_p
	     << endl;
    }

    // Print out NBOT shortest perturbed time steps.

    cerr << endl << "  " << NBOT << " shortest perturbed time steps"
	 << " (elder binary component only):" << endl << endl;

    qsort((void *)list_dt,  (size_t)np, sizeof(dt_pair), compare_dt);

    for (i = 0; i < NBOT; i++)
	fprintf(stderr, "    %3d.     %-15s  %.4e\n",
		i+1,
		list_dt[i].b->format_label(),
		list_dt[i].dt);

    // Print out top NTOP particles in (1) instantaneous cost (steps),
    //				 and (2) integrated cost since the last output.
    // Expand soon to include force calculations...

    // Sort the arrays in order of decreasing count:

    qsort((void *)steps_inst,  (size_t)np, sizeof(count_pair), compare_steps);
    qsort((void *)steps_delta, (size_t)n,  sizeof(count_pair), compare_steps);
    qsort((void *)f_dir_delta, (size_t)nd, sizeof(count_pair), compare_steps);
    qsort((void *)f_ind_delta, (size_t)ni, sizeof(count_pair), compare_steps);

    cerr << endl << "  Top " << NTOP
	 << " nodes by instantaneous (excl. unperturbed)"
	 << " and delta (all) steps:"
	 << endl << endl;

    fprintf(stderr, "             %-26s      %-30s\n",
	    "--- instantaneous ---",
	    "------- delta -------");
    fprintf(stderr, "            %-26s      %-30s\n\n",
	    "(normalized to dt = 1) ",
	    "(since last log output)");

    char tmp[128];

    for (i = 0; i < NTOP; i++) {

	if (i < np)
	    sprintf(tmp, "%3d.     %-10s %10.0f",
		    i+1,
		    steps_inst[i].b->format_label(),
		    steps_inst[i].count);
	else
	    strcpy(tmp, " ");
	fprintf(stderr, "    %-35s", tmp);

	if (i < n)
	    sprintf(tmp, "%-10s %10.0f",
		    steps_delta[i].b->format_label(),
		    steps_delta[i].count);
	else
	    strcpy(tmp, " ");
	fprintf(stderr, "      %-30s\n", tmp);
    }
    cerr << endl << "  total delta_steps = " << delta_steps_tot << endl;

    cerr << endl << "  Top " << NTOP
	 << " nodes by delta(force calculations):"
	 << endl << endl;

    fprintf(stderr, "             %-26s      %-30s\n",
	    "------- direct ------",
	    "------ indirect -----");

    for (i = 0; i < NTOP; i++) {

	if (i < nd)
	    sprintf(tmp, "%3d.     %-10s %10.0f",
		    i+1,
		    f_dir_delta[i].b->format_label(),
		    f_dir_delta[i].count);
	else
	    strcpy(tmp, " ");
	fprintf(stderr, "    %-35s", tmp);

	if (i < ni && f_ind_delta[i].count > 0)
	    sprintf(tmp, "%-10s %10.0f",
		    f_ind_delta[i].b->format_label(),
		    f_ind_delta[i].count);
	else
	    strcpy(tmp, " ");
	fprintf(stderr, "      %-30s\n", tmp);
    }
    cerr << endl << "  total delta_f_dir = " << delta_f_dir_tot
	 << "  delta_f_ind = " << delta_f_dir_tot
	 << endl;

    cerr.precision(PRECISION);

    delete [] steps_inst;
    delete [] steps_delta;
    delete [] f_dir_delta;
    delete [] f_ind_delta;
}

local void print_dstar_params(dyn* b)		// b is binary center of mass
    {
    if (((hdyn*)b)->get_use_dstar())
	print_binary_dstars(b);
}

local bool print_dstar_stats(dyn* b, bool mass_function,    // b is binary 
			     vector center, bool verbose)   // center of mass
{
  if (((hdyn*)b)->get_use_dstar()) {
    dstar_stats(b, mass_function, center, verbose);

    return true;
  }

  return false;
}

void print_statistics(hdyn* b)
{
    // General system statistics (omit time output
    // and O(N^2) operations):

    sys_stats(b,
	      0.5,	// energy cutoff
	      true,	// verbose
	      true,	// binaries
	      2,	// which lagr
	      false,	// print time
	      false,	// compute energy
	      false,	// allow n^2 ops
	      print_dstar_params,
	      print_dstar_stats);

    // Statistics on time steps and bottlenecks:
    
    timestep_stats(b);
}

local int n_bound(hdyn* b, vector& cod_vel)
{
    int nb = 0;
    for_all_daughters(hdyn, b, bb)
	if (0.5*square(bb->get_vel()-cod_vel) + bb->get_pot() < 0)
	    nb += bb->n_leaves();
    return nb;
}

local real m_bound(hdyn* b, vector& cod_vel)
{
    real mb = 0;
    for_all_daughters(hdyn, b, bb)
	if (0.5*square(bb->get_vel()-cod_vel) + bb->get_pot() < 0)
	    mb += bb->get_mass();
    return mb;
}

local void get_n_and_m_bound(hdyn* b, vector& cod_vel, int& nb, real& mb)
{
    nb = 0;
    mb = 0;
    for_all_daughters(hdyn, b, bb)
	if (0.5*square(bb->get_vel()-cod_vel) + bb->get_pot() < 0) {
	    nb += bb->n_leaves();
	    mb += bb->get_mass();
    }
}



void update_cpu_counters(hdyn * b)
{
    real last_cpu = b->get_kira_counters()->cpu_time;
    b->get_kira_counters()->cpu_time = cpu_time();
    b->get_kira_counters()->total_cpu_time += (cpu_time() - last_cpu);
    write_counters_to_log(b);
}

void log_output(hdyn * b, real steps, real eps, kira_counters* kc_prev)
{
    // Standard log output (to stderr, note).

    vector cod_pos, cod_vel;
    compute_com(b, cod_pos, cod_vel);

    compute_densities(b, cod_pos, cod_vel);	// does nothing unless
						// HARP is present

    int n_bound;
    real m_bound;
    get_n_and_m_bound(b, cod_vel, n_bound, m_bound);
    
    cerr << endl;
    for (int k = 0 ; k < 40; k++) cerr << '-';
    cerr << "\n\nTime = " << b->get_system_time()
	 << "  N = "    << b->n_leaves() << " (" << n_bound << " bound)"
	 << "  mass = " << total_mass(b) << " (" << m_bound << " bound)"
	 << "  steps = " << steps << endl;

    if (b->get_use_sstar())
	print_sstar_time_scales(b);

    cerr.precision(10);
    print_recalculated_energies(b, eps * eps, true, true);
    cerr.precision(PRECISION);

    if (steps > 0) {
	update_cpu_counters(b);
	print_counters(b->get_kira_counters(), kc_prev);
    }

    print_statistics(b);
    flush(cerr);
}

void snap_output(hdyn * b, real steps, int& snaps,
		 bool reg_snap, bool last_snap,
		 char * snap_save_file,
		 real t, real ttmp, real t_end,
		 real& t_snap, real dt_snap, bool verbose)
{
    // Save a snapshot.

    update_cpu_counters(b);
    b->set_time(t);

    if (last_snap) {

	// Write (overwrite) snap_save_file.

	ofstream file(snap_save_file);

	if (file) {
	    put_node(file, *b);
	    file.close();
	    cerr << "Snapshot saved in file "
		 << snap_save_file
		 << " (save CPU = "
		 << cpu_time() - b->get_kira_counters()->cpu_time
		 << ")\n";
	} else {
	    if (snap_save_file == NULL)
		cerr << "No save file specified.\n";
	    else
		cerr << "Error opening save file "
		     << snap_save_file << "\n";
	}
    }

    if (reg_snap) {

	if (verbose) cerr << endl;
	    cerr << "Snapshot output #" << ++snaps
		 << " at steps = " << steps
		 << ", time = " << b->get_system_time() << endl;
	if (verbose) cerr << endl;

	put_node(cout, *b);
	cout << flush;

	if (ttmp > t_end) exit(0);
    }
}
