
       //=======================================================//    _\|/_
      //  __  _____           ___                    ___       //      /|\
     //  /      |      ^     |   \  |         ^     |   \     //          _\|/_
    //   \__    |     / \    |___/  |        / \    |___/    //            /|\
   //       \   |    /___\   |  \   |       /___\   |   \   // _\|/_
  //     ___/   |   /     \  |   \  |____  /     \  |___/  //   /|\     
 //                                                       //            _\|/_
//=======================================================//              /|\

// Implement run-time corrections/adjustments to kira.

#include <star/sstar_to_dyn.h>	// These are needed for ifstream to be visible.
#include <star/dstar_to_dyn.h>	// I don't know why...  (SLWM 3/98)

#include "hdyn.h"

local void clean_up_file(char* name)
{
    char tmp[80];
    sprintf(tmp, "rm %s", name);
    system(tmp);
    cerr << "deleted file " << name << endl;
}

#define N_FILES 9

void clean_up_files()
{

    char * files[N_FILES] = {"ABORT", "DUMP", "ENERGY", "KILL_MULTIPLES",
			     "LOG", "MULTIPLES", "PARAMS", "PP", "STOP"};

    // Clean up...

    for (int i = 0; i < N_FILES; i++) {
	ifstream file(files[i]);
	if (file) {
	    file.close();
	    clean_up_file(files[i]);
	}
    }
}

bool check_file(char* name, bool del)
{
    ifstream file(name);
    if (file) {
	file.close();
	if (del) clean_up_file(name);
	return true;
    } else
	return false;
}

local void print_multiples(hdyn * b)
{
    cerr << "\n\nIn print_multiples:\n";
    int n_mult = 0;

    for_all_daughters(hdyn, b, bb)
	if (!bb->is_leaf())
	    if (bb->n_leaves() > 2) {
		n_mult++;
		cerr << "\nFound multiple system " << bb->format_label()
		     << endl;
		real eb = print_structure_recursive(bb);
		cerr << "Energy = " << eb << endl;
	    }

    if (n_mult == 0) cerr << "    (no multiples found)\n";

    cerr << endl;
}

local void force_step(hdyn * b, real t)
{
    real dt = b->get_timestep();
    int i = 0;
    while (b->get_next_time() > t && i++ < 10)
	b->set_timestep((dt /= 2));
    b->set_timestep((dt *= 2));
}

local void force_unperturbed_step(hdyn * b)
{
    kepler * kep = b->get_kepler();
    if (kep) {
	real dt_unp = b->get_unperturbed_timestep();
	while (b->get_next_time() > b->get_parent()->get_next_time())
	    b->set_unperturbed_timestep((dt_unp -= kep->get_period()));
	b->set_unperturbed_timestep((dt_unp += kep->get_period()));
    } else {
	cerr << "Warning: " << b->format_label()
	     << " has no kepler pointer\n";
	force_step(b, b->get_parent()->get_next_time());
    }
}

local void adjust_vel(hdyn * b)
{
    real v = abs(b->get_vel());
    real r = abs(b->get_pos());

    b->set_vel(-v * b->get_pos() / r);	// Preserve v

    hdyn * s =  b->get_binary_sister();
    real mass_fac = -b->get_mass()/s->get_mass();

    s->set_vel(b->get_vel() * mass_fac);

    // Recompute acc and jerk (acc unchanged)...

    vector a_2b, j_2b;
    real p_2b;
    real d_min_sister = VERY_LARGE_NUMBER;
    hdyn * p_nn_sister = NULL;
    real eps2 = 0.0;
    b->calculate_partial_acc_and_jerk(b->get_parent(), b->get_parent(), b,
				      eps2,
				      a_2b, j_2b, p_2b,
				      d_min_sister, p_nn_sister,
				      false, NULL, b);

//  b->set_acc_and_jerk_and_pot(mass_fac*a_2b, mass_fac*j_2b, -mass_fac*p_2b);

}

local bool kill_multiple(hdyn * b)
{
    // For now, adjust eccentricity of top-level node b only...

    hdyn * od = b->get_oldest_daughter();
    kepler * kep = od->get_kepler();

    if (kep) {

	if (kep->get_eccentricity() < 1) {

	    // Unperturbed system.  Reduce timesteps to advance
	    // system in scheduling, then modify kepler structure.

	    // Top-level CM step...

	    force_step(b, b->get_system_time());

	    // Top-level binary step...

	    force_unperturbed_step(od);

	    // Lower levels, if any...

	    for_all_nodes(hdyn, b, bb) {
		if (bb->get_parent() != b && bb->get_elder_sister() == NULL) {

		    if (bb->get_kepler() == NULL) {

			cerr << "Warning: " << bb->format_label()
			     << " has no kepler pointer\n";

			force_step(bb, bb->get_parent()->get_next_time());

		    } else

			force_unperturbed_step(bb);

		}
	    }

	    // Modify top-level orbit....

	    kep->set_eccentricity(0.99999);

	    // Probably need to do more (reinitialize) here...	<<<<<<

	    return true;
	}
	
    } else {

	// Force CM and top-level steps only...

	force_step(b, b->get_system_time());
	force_step(b->get_oldest_daughter(), b->get_system_time());

	// Modify relative velocity of components.

	adjust_vel(b->get_oldest_daughter());

	return true;

    }

    return false;
}

local bool kill_multiples(hdyn * b)
{

    //------------------------------------------------------------
    cerr << "\n\nkill_multiples: not yet implemented...\n\n";
    return false;
    //------------------------------------------------------------

    bool killed = false;
    for_all_daughters(hdyn, b, bb)
	if (!bb->is_leaf())
           if (bb->n_leaves() > 2) {
		cerr << "\nFound multiple system " << bb->format_label()
		     << endl;
		print_structure_recursive(bb);
		if (kill_multiple(bb)) {
		    cerr << "...killed\n";
		    killed = true;
		}
	    }

    return killed;
}

local void modify_params(hdyn * b, char * name, 
			 real& eta, real& lag_factor, real& t_end)
{
    ifstream file(name);
    if (file) {

	cerr << "\n\n*** Reading parameter changes from file " << name
	     << "\n\n";

	// File should contain lines of the form:
	//
	//		-keyword value
	//
	// where the keywords a, C, f, g, k, N, and t are permitted
	// (meanings are the same as if specified on the command line,
	// except that "-t" increments t_end).
	//
	// Parameter changes take place IMMEDIATELY.

	char *s, line[128];

	while (file.get(line, 128, '\n')) {

	    // NOTE: If get() found the terminating '\n', it left it
	    // in the input stream as the first unread character (a C++
	    // feature, not a bug).  To avoid an infinite loop, read
	    // that character now.  Partial reads of lines probably
	    // mean an error, so we simply read characters until the
	    // terminator is found.

	    char c;
	    while (file.get(c) && c != '\n') ;

	    // Parse the line.

	    if (s = strstr(line, "-a")) {

		real tmp = atof(s+2);
		if (tmp > 0 && tmp < 1) {
		    cerr << "Setting eta = " << tmp << endl;
		    eta = tmp;
		}

	    } else if (s = strstr(line, "-C")) {

		real tmp = atof(s+2);
		if (tmp > 0 && tmp < 1800) {
		    cerr << "Setting harp_max_cpu = " << tmp << endl;
		    kira_set_harp_cpu(tmp);
		}

	    } else if (s = strstr(line, "-f")) {

		real tmp = atof(s+2);
		if (tmp > 0 && tmp < 100) {
		    cerr << "Setting d_min = " << tmp << " / N\n";
		    tmp /= b->n_leaves();
		    b->set_d_min_sq(tmp*tmp);
		}

	    } else if (s = strstr(line, "-g")) {

		real tmp = atof(s+2);
		if (tmp > 0 && tmp < 100) {
		    cerr << "Setting lag_factor = " << tmp << " (squared)\n";
		    lag_factor = tmp*tmp;
		}

	    } else if (s = strstr(line, "-k")) {

		real tmp = atof(s+2);
		if (tmp > 0 && tmp < 1) {
		    cerr << "Setting gamma = " << tmp << endl;
		    b->set_gamma2(tmp*tmp);
		}

	    } else if (s = strstr(line, "-N")) {

		int tmp = atoi(s+2);
		if (tmp > 0) {
		    cerr << "Setting n_check = " << tmp << endl;
		    set_n_check(tmp);
		}

	    } else if (s = strstr(line, "-t")) {

		real tmp = atof(s+2);
		t_end += tmp;
		cerr << "Setting t_end = " << t_end << " / N\n";

	    }
	}

	file.close();
	clean_up_file(name);

	cerr << endl;
    }
}

local void pp(hdyn * b, char * name)
{
    ifstream file(name);
    if (file) {

	cerr << "\n\n*** Running pp3 on particle(s) listed in file " << name
	     << ":\n\n";

	// File should contain valid particle names, one per line.

	char *s, line[128];

	while (file.get(line, 128, '\n')) {

	    // NOTE: If get() found the terminating '\n', it left it
	    // in the input stream as the first unread character (a C++
	    // feature, not a bug).  To avoid an infinite loop, read
	    // that character now.  Partial reads of lines probably
	    // mean an error, so we simply read characters until the
	    // terminator is found.

	    char c;
	    while (file.get(c) && c != '\n') ;

	    // Add a trailing '\0' to the input line at the first
	    // non-printing location.

	    bool ok = false;
	    for (int i = 0; i < 128; i++) {
		if (line[i] < ' ') {
		    line[i] = '\0';
		    ok = true;
		    break;
		}
	    }

	    // (Silently ignore unterminated lines.)

	    if (ok) {

		// Parse the line.

		hdyn* bb = (hdyn*) node_with_name(line, b->get_root());

		if (!bb)
		    cerr << "Unable to find node " << line << endl;
		else
		    pp3(bb);
	    }
	}

	file.close();
	clean_up_file(name);

	cerr << endl;
    }
}

local void dump_to_file(hdyn* b, char* name)
{
    ifstream file(name);
    bool written = false;
    if (file) {
	char dumpfile[128];
	if (file.get(dumpfile, 128, '\n')) {
	    ofstream dump(dumpfile);
	    if (dump) {
		put_node(dump, *b);
		dump.close();
		cerr << "Data written to file " << dumpfile << endl;
		written = true;
	    }
	}
	file.close();
	clean_up_file(name);
    }
    if (!written)
	cerr << "No data written" << endl;
}

void check_runtime(hdyn* b,
		   real eps, real& eta, real& lag_factor, real& t_end,
		   bool& tree_changed)
{
    // Note the order in which we check things -- ABORT at end allows
    // us to get other output before quitting.

    if (check_file("PARAMS", false))
	modify_params(b, "PARAMS", eta, lag_factor, t_end);

    if (check_file("PP", false))
	pp(b, "PP");

    if (check_file("DUMP", false)) {
	cerr << "\n\n(Unsynchronized system dump forced by user)\n\n";
	cerr << "Time = " << b->get_system_time()
	     << "  N = "    << b->n_leaves()
	     << "  mass = " << total_mass(b) << endl;
	dump_to_file(b, "dump");
	cerr << endl;
    }

    if (check_file("ENERGY")) {
	cerr << "\n\n(Energy output forced by user)\n\n";
	cerr << "Time = " << b->get_system_time()
	     << "  N = "    << b->n_leaves()
	     << "  mass = " << total_mass(b) << endl;
	print_recalculated_energies(b, eps * eps);
	cerr << endl;
    }

    if (check_file("LOG")) {
	cerr << "\n\n(Log output forced by user)\n\n";
	cerr << "Time = " << b->get_system_time()
	     << "  N = "    << b->n_leaves()
	     << "  mass = " << total_mass(b) << endl;
	print_recalculated_energies(b, eps * eps);
	print_counters(b->get_kira_counters());
	print_statistics(b);
	cerr << endl;
    }

    if (check_file("MULTIPLES")) {
	cerr << "\n\n(Multiple output forced by user)\n\n";
	cerr << "Time = " << b->get_system_time()
	     << "  N = "    << b->n_leaves()
	     << "  mass = " << total_mass(b) << endl;
	print_multiples(b);
	cerr << endl;
    }

    if (check_file("KILL_MULTIPLES"))
	tree_changed |= kill_multiples(b);

    if (check_file("ABORT")) {
	cerr << "\n\nAbort forced by user at time "
	     << b->get_system_time() << "\n\n" << flush;
	clean_up_files();
	exit(0);
    }
}
