//
// NODE_TT.C : basic tree handling package
//

#include "node.h"

node *  mk_flat_tree(int n, npfp new_node_type, hbpfp a_hbpfp, sbpfp a_sbpfp)
    {
    node * root, * by, * bo;

    root = (* new_node_type)(a_hbpfp, a_sbpfp);
    bo = (* new_node_type)(a_hbpfp, a_sbpfp);
    root->set_oldest_daughter(bo);
    bo->set_parent(root);

    for (int i = 1; i < n; i++)
        {
        by = (* new_node_type)(a_hbpfp, a_sbpfp);
        bo->set_younger_sister(by);
        by->set_elder_sister(bo);
	by->set_parent(root);
        bo = by;
        }

    return root;
    }

char * string_index_of_node(node * n)
{
    char * tmp;
    tmp = n->get_name();
    if(tmp  != NULL){
	return tmp;
    }else{
	static char  integer_id[20];
	sprintf(integer_id,"%d",n->get_index());
	n->set_label(integer_id);
	return n->get_name();
    }
}

char * construct_label(node * ni, node * nj)
{
    static char new_name[256];
    sprintf(new_name,"(%s, %s)",string_index_of_node(ni),
	    string_index_of_node(nj));
    return new_name;
}

int depth_of_node(node * ni)
{
    int depth = 0;
    while((ni = ni->get_parent()) != NULL)depth ++;
    return depth;
}

// is_descendent_of: return TRUE if child is among the offspring of parent.
//                   mode = 1 ==> include parent in "offspring" list
//                   mode = 0 ==> exclude parent from consideration
//
int is_descendent_of(node *child, node *parent, int mode)
{
    if (mode == 0 && child == parent) return 0;

    while (child != NULL) {
	if(child == parent) {
	    return 1;
	} else {
	    child = child->get_parent();
	}
    }
    return 0;
}

node * common_ancestor(node * ni, node * nj)
{
    int i;
    real difference = depth_of_node(ni) - depth_of_node(nj);
    if(difference > 0){
	for(i = 0; i<difference; i++) ni = ni->get_parent();
    }else if (difference < 0){
	for(i = 0; i<-difference; i++) nj = nj->get_parent();
    }
    while(ni != nj){
	ni = ni->get_parent();
	nj = nj->get_parent();
    }
    return ni;
}

node * node_with_index(int i, node * root)
{
    node * n = root;
    if(n->get_index() == i) return n;
    if(n->get_oldest_daughter() !=NULL){
	for(node * nn = n->get_oldest_daughter();nn != NULL;
	    nn = nn->get_younger_sister()){
	    node * tmp = node_with_index(i,nn) ;
	    if(tmp != NULL)return tmp;
	}
    }
    return NULL;
}

node * pointer_to_sister(node & n)
{
    node * sister;
    if(n.get_elder_sister() != NULL){
	sister= n.get_elder_sister();
    }else{
	sister = n.get_younger_sister();
    }
    if(sister == NULL){
	cerr << "Error: sister of " ;
	n.print_label(cerr);
	cerr << " is NULL\n";
	exit(-1);
    }
    return sister;
}


// delete_node_from_general_tree
// delete node n. Do not check if the parent has more than 2 remaining
// daughters or not.


void delete_node_from_general_tree(node & n)
{
    if(&n == NULL){
	cerr << "Warning: delete_node_from_general_tree, n is null\n";
	return;
    }

    node *  parent = n.get_parent();
    
    // check if n is head without parent or sisters
    if(parent == NULL){
	cerr << "Warning: delete_node_from_general_tree, n has no parent\n";
	return;
    }

    n.set_parent(NULL);

    node *  elder_sister = n.get_elder_sister();
    node *  younger_sister = n.get_younger_sister();

    if (parent->get_oldest_daughter() == &n){
	parent->set_oldest_daughter(younger_sister);
    }

    if(elder_sister != NULL){
	elder_sister->set_younger_sister(younger_sister);
    }
    if(younger_sister != NULL){
	younger_sister->set_elder_sister(elder_sister);
    }

}


void remove_node_with_one_daughter(node &n)
{
    if(n.get_oldest_daughter()->get_younger_sister()!=NULL){
	cerr << "remove_node_with_one_daughter: daughters of n is "
	     << "larger than 1\n";
	exit(1);
    }
    node *  daughter = n.get_oldest_daughter();
    if (daughter->get_elder_sister() != NULL ||
	daughter->get_younger_sister() != NULL ){
	cerr << "remove_node_with_one_daughter: daughter of n has  sisters\n";
	exit(1);
    }

    node *  parent = n.get_parent();
    
    // check if n is head without parent or sisters
    if(parent == NULL){
	cerr << "Warning: remove_node_with_one_daughter, n has no parent\n";
	return;
    }


    node *  elder_sister = n.get_elder_sister();
    node *  younger_sister = n.get_younger_sister();

    if (parent->get_oldest_daughter() == &n){
	parent->set_oldest_daughter(daughter);
    }

    if(elder_sister != NULL){
	elder_sister->set_younger_sister(daughter);
    }
    if(younger_sister != NULL){
	younger_sister->set_elder_sister(daughter);
    }

    daughter->set_parent(parent);
    daughter->set_younger_sister(younger_sister);
    daughter->set_elder_sister(elder_sister);
}

// delete_node_from_binary_tree
// delete node n and repair the tree

void delete_node_from_binary_tree(node & n)
{
    if(&n == NULL){
	cerr << "Warning: delete_node_from_binary_tree, n is null\n";
	return;
    }

    node *  parent = n.get_parent();

    delete_node_from_general_tree(n);
    remove_node_with_one_daughter(*parent);
}
    

// extend_tree
// extend a tree by inserting a new node to a location presently
// occupied by old_n. old_n becomes the only child of the newly
// inserted node. This function returns the address of the
// newly created node
//

node *  extend_tree(node & old_n)
{
    if(&old_n == NULL){
	cerr << "Warning:extend_tree, old_n is null\n";
	return NULL;
    }

    //check if old_n is root
    if(old_n.get_parent() == NULL){
	cerr << "extend_tree, old_n is root, cannot insert\n";
	exit(1);
    }

    node *  new_n = new node();

    // set the pointer of ex-parent of old_n if she was the oldest daughter
    node *  parent = old_n.get_parent();
    if (parent->get_oldest_daughter() == (&old_n)){
	// old_n is the oldest daughter of her parent
	parent->set_oldest_daughter(new_n);
    }
    
    // set the pointers of ex-sisters of old_n
    node *  elder = old_n.get_elder_sister();
    if(elder != NULL){
	elder->set_younger_sister(new_n);
    }
    node *  younger = old_n.get_younger_sister();
    if(younger != NULL){
	younger->set_elder_sister(new_n);
    }
    
    new_n->set_parent(old_n.get_parent());
    new_n->set_elder_sister(old_n.get_elder_sister());
    new_n->set_younger_sister(old_n.get_younger_sister());
    new_n->set_oldest_daughter(&old_n);

    old_n.set_elder_sister(NULL);
    old_n.set_younger_sister(NULL);
    old_n.set_parent(new_n);

    return new_n;
}



// add_node
// insert n into the tree as the oldest_daughter of the
// parent. The ex-oldest node of the parent becomes the
// younger sister of n.

void add_node(node &n, node & parent)
{
    if(&n == NULL){
	cerr << "Warning:add_node, n is null\n";
	return;
    }

    if(&parent == NULL){
	cerr << "Warning:add_node, parent is null\n";
	return;
    }

//    if(parent.isphysical()){
//	cerr << "Error:add_node, parent is a single particle\n";
//	exit(1);
//    }

// The following part test if n is completely isoleted
#if 0
    // check if n is head without parent or sisters
    if(n.get_parent() != NULL){
	cerr << "add_node, n has parent\n";
	exit(1);
    }
    if(n.get_elder_sister() != NULL){
	cerr << "add_node, n has an elder sister\n";
	exit(1);
    }
    if(n.get_younger_sister() != NULL){
	cerr << "add_node, n has an younger sister\n";
	exit(1);
    }
#endif

    // set the pointers of ex-oldest-daughter of parent
    node *  ex = parent.get_oldest_daughter();
    if(ex != NULL){
	ex->set_elder_sister(&n);
    }


    parent.set_oldest_daughter(&n);

    n.set_elder_sister(NULL);
    n.set_younger_sister(ex);
    n.set_parent(&parent);
}
    

//insert_node_binary_tree : put in n into the tree by creating
// new node at the location of old_n and making both old_n and
// n its daughters;
void insert_node_into_binary_tree(node &n, node & old_n)
{
    node *  new_n = extend_tree(old_n);
    add_node(n, *new_n);
}

