//
//  mkdouble.C: construct a linked list of unit-mass nodes.
//
//	       Simon Portegies Zwart, July 1996
//

#include "node.h"
#include "double_star.h"
//#include "single_star.h"
#include "main_sequence.h"
#include "dstar_to_dyn.h"


void add_secondary(node* original, real mass_ratio) {

    node* primary = new node;
    node* secondary = new node;

    // Add new links.

    original->set_oldest_daughter(primary);

    primary->set_parent(original);
    secondary->set_parent(original);

    primary->set_younger_sister(secondary);
    secondary->set_elder_sister(primary);

    // Set new masses.

    primary->set_mass(original->get_mass());
    secondary->set_mass(mass_ratio*original->get_mass());
    original->inc_mass(secondary->get_mass());

    // Naming convention:

    if (original->get_name() == NULL)
        if (original->get_index() >= 0) {
            char tmp[64];
            sprintf(tmp, "%d", original->get_index());
            original->set_name(tmp);
        }

    primary->set_name(original->get_name());
    secondary->set_name(original->get_name());
    strcat(primary->get_name(), "a");
    strcat(secondary->get_name(), "b");

   }

void mksecondary(node* b, real binary_fraction, real lower_limit) {

    // For now, use a flat distribution in secondary mass ratio.
    // Assume that the probability of a star being the primary of
    // a binary is independent of mass.

    real sum = 0;
    b->set_mass(0);

    for_all_daughters(node, b, bi) {
        sum += binary_fraction;
        if (sum >= 1) {
            sum -= 1;

            real mass_ratio = randinter(lower_limit, 1);        // Quick fix...
            add_secondary(bi, mass_ratio);

        }
        b->inc_mass(bi->get_mass());
    }
}

local void  evolve_binary(node* the_binary, real end_time) {

      real time = 0;   
      real dt=0;
      if (!the_binary->is_root())
         if (the_binary->get_parent()->is_root()) {
             cerr<<"binary "<<the_binary<<endl;
             do {
                dt = the_binary->get_starbase()->get_evolve_timestep();
                time += max(0., min(end_time-time, dt));
                cerr<<"dt, t= " << dt<<" " << time<<endl;
                the_binary->get_starbase()->evolve_element(time);
             }
	     while (time<end_time);
         }
   }

void main(int argc, char ** argv) {

    bool m_flag = false;

    bool reandom_initialization = false;
    int  n = 1;
    int id=1;
    real mf=1,rf=1, tf=1;
    stellar_type type = Main_Sequence;
    binary_type bin_type = Detached;
    real binary_fraction = 1.0;
//		Initial conditions from Tutukov & Yunglesson 1993.
    real sma    = 138;
    real ecc    = 0.0;
    real m_prim = 13.1;
    real m_sec  =  9.8;
    real mass_ratio = 0.75;
    real lower_limit = 0.0;
    int random_seed = 0;
    char seedlog[64];
    real start_time = 0;
    real end_time   = 35;
    extern char *poptarg;
    int c;
    char* param_string = "a:e:M:m:n:q:s:T:";

    while ((c = pgetopt(argc, argv, param_string)) != -1)
	switch(c) {
            case 'a': sma = atof(poptarg);
                      break;
            case 'e': ecc = atof(poptarg);
                      break;
            case 'M': m_prim = atof(poptarg);
                      break;
            case 'm': m_flag = true;
		      m_sec = atof(poptarg);
                      break;
	    case 'n': n = atoi(poptarg);
		      break;
            case 'q': mass_ratio = atof(poptarg);
                      break;
            case 'T': end_time = atof(poptarg);
                      break;
            case 's': random_seed = atoi(poptarg);
                      break;
            case '?': params_to_usage(cerr, argv[0], param_string);
		      exit(1);
	}

    if (n <= 0) err_exit("mknodes: N > 0 required!");
    int actual_seed = srandinter(random_seed);
    sprintf(seedlog, "         random number generator seed = %d",actual_seed);

    mf = 1./m_prim;
    if (m_flag) 
       mass_ratio = m_sec / m_prim;

    // Create flat tree 
    node *root  = mknode(n);
//    node *root  = mknode(n);
    root->log_history(argc, argv);
    root->log_comment(seedlog);
    root->get_starbase()->set_stellar_evolution_scaling(mf, rf, tf);
//    node* the_binary = root->get_oldest_daughter();
    node* the_binary = root->get_oldest_daughter();

//		Add the stars and secondary.
    add_secondary(the_binary, mass_ratio);
    addstar(root, start_time, type);
    double_star* new_double = 
		 new_double_star(the_binary, sma, ecc, 
				 start_time, id, bin_type);
    put_node(cout, *root);

//	Test pointer structure
    cerr<<"est pointer structure"<<endl;
    node *b = root->get_oldest_daughter();
    starbase *s = b->get_starbase();
    star *st     = (star*)b->get_starbase();
    cerr<<"binary+s:" << b<<" "<<s<<" " <<st<<endl;
    cerr<<"a e m"<<s->get_effective_radius() <<" "
               <<s->get_total_mass()<<endl;
    cerr<<"p, s"<< st->get_primary()<<" "<<st->get_secondary() <<endl;
    cerr<<"M R, m r"<<st->get_primary()->get_total_mass() << " " 
                <<st->get_primary()->get_effective_radius() << " "
                <<st->get_secondary()->get_total_mass() << " " 
                <<st->get_secondary()->get_effective_radius()<< endl;
    cerr<<"companion p, s: "<< 
         st->get_secondary()->get_companion()<<" " 
       <<st->get_primary()->get_companion()<<endl;
   cerr<<"companion p, s: "<<st->get_companion(st->get_secondary())<<" "
                           <<st->get_companion(st->get_primary())<< endl; 
   
    evolve_binary(the_binary, end_time);
    put_node(cout, *root);
}
