#include "node.h"
#include "double_star.h"
#include "main_sequence.h"

#define REPORT_ADD_DOUBLE        false
#define REPORT_DEL_DOUBLE        false
#define REPORT_EVOLVE_DOUBLE     false

void add_secondary(node* original, real mass_ratio) {

    node* primary = new node;
    node* secondary = new node;

    // Add new links.

    original->set_oldest_daughter(primary);

    primary->set_parent(original);
    secondary->set_parent(original);

    primary->set_younger_sister(secondary);
    secondary->set_elder_sister(primary);

    // Set new masses.

    primary->set_mass(original->get_mass());
    secondary->set_mass(mass_ratio*original->get_mass());
    original->inc_mass(secondary->get_mass());

    // Naming convention:

    if (original->get_name() == NULL)
        if (original->get_index() >= 0) {
            char tmp[64];
            sprintf(tmp, "%d", original->get_index());
            original->set_name(tmp);
        }

    primary->set_name(original->get_name());
    secondary->set_name(original->get_name());
    strcat(primary->get_name(), "a");
    strcat(secondary->get_name(), "b");

   }

void mksecondary(node* b, real binary_fraction, real lower_limit) {

    // For now, use a flat distribution in secondary mass ratio.
    // Assume that the probability of a star being the primary of
    // a binary is independent of mass.

    real sum = 0;
    b->set_mass(0);

    for_all_daughters(node, b, bi) {
        sum += binary_fraction;
        if (sum >= 1) {
            sum -= 1;

            real mass_ratio = randinter(lower_limit, 1);        // Quick fix...
            add_secondary(bi, mass_ratio);

        }
        b->inc_mass(bi->get_mass());
    }
}

real random_exponential_mass(const real m_min,
				   const real m_max,
				   const real m_alpha) {

  real random    = randinter(0., 1.);
  real m_const   = pow(m_min, 1-m_alpha)
    - pow(m_max, 1-m_alpha);
  real m_prim    = pow(random*m_const
		       + pow(m_max, 1-m_alpha), 1/(1-m_alpha));

  return m_prim;
}


void mkrandom_binary(const real lm_min, const real lm_max,
		     const real m_exp,
		     const real la_min, const real la_max,
		     const bool e_flag,
		     real &m_prim, real &m_sec, real &semi,
		     real &ecc) {

  // Primary mass from IMF (flat in log m_prim)
  real mmin = pow(10, lm_min);
  real mmax = pow(10, lm_max);
  m_prim = random_exponential_mass(mmin, mmax, m_exp);

  // Initial secondary mass (selected between m_min and m_prim
  // with equal probability per unit mass.
  m_sec  = randinter(mmin, m_prim);

  // The Initial orbital separation is chosen flat in log a.
  semi = pow(10., randinter(la_min, la_max));

  // Zero eccentricity for all binaries.
  if (e_flag)
    ecc = sqrt(randinter(0.0, 1.0));
  else
    ecc = 0;
  
  cerr << m_prim <<" "
       << m_sec <<" "
       << semi <<" "
       << ecc
       << endl;
}


void  adddouble(node * b, real dyn_time,
		binary_type type,		// Defaults set in
		bool random_initialization,	// double_star.h
		real a_min, real a_max,
		real e_min, real e_max)
{
  if (REPORT_ADD_DOUBLE) {
    cerr.precision(15);
    cerr<<"adddouble: "<<b<<" "<<dyn_time;
    cerr.precision(6);
    cerr <<" "<<a_min<<" "<<a_max<<" "<<e_min<<" "<<e_max << endl;
  }
    
    real stellar_time = b->get_starbase()->conv_t_dyn_to_star(dyn_time);
    addstar(b, stellar_time);

    real ecc, sma;
    // binary_type local_type = Unknown_Binary_Type;

    int id;
    real a_const = log(a_max) - log(a_min);
    for_all_nodes(node, b, bi) {
	if (bi->is_parent() && !bi->is_root()) {
            story * s = b->get_starbase()->get_star_story();
//            extract_story_chapter(local_type, sma, ecc, *s);
            b->get_starbase()->set_star_story(NULL);
            delete s;

            id = bi->get_index();

            if (random_initialization) {
               sma = a_min*exp(randinter(0., a_const));
               do {
                  ecc = sqrt(randinter(0., 1.));
               }
               while (ecc<e_min && ecc>=e_max);
            }
	    else { //if (local_type==Unknown_Binary_Type)
//               type = local_type;
//            else {
               id = bi->get_index();
               sma = a_min*exp(randinter(0., a_const));
               do {
                   ecc = sqrt(randinter(0., 1.));
               }
               while (ecc<e_min && ecc>=e_max);
            }
	    if (REPORT_ADD_DOUBLE) 
	      cerr<<"adddouble: iae: "<<id<<" "<<sma<<" "<<ecc<<endl;
	    
            double_star* new_double = new_double_star(bi, sma, ecc, stellar_time, id,
                         type);
	    if (REPORT_ADD_DOUBLE) {
	      cerr<<"double: "<<new_double<<endl;
	      new_double->dump(cerr);
	      put_state(make_state(new_double));
	    }
         }
      }
   }

