
// kira.h: copied from hdyn/evolve -- necessary to define kira_stats.

#include "hdyn.h"

// Counters for kira:

typedef unsigned long step_t;

typedef struct {

    real cpu_time;		  // CPU time since this process started
    real total_cpu_time;	  // total CPU time for the entire calculation

    step_t step_top_single;	  // top-level single steps
    step_t step_top_cm;		  // top-level CM steps
    step_t step_low_level;	  // low-level steps
    step_t force_correction;	  // force corrections

    step_t pert_step;		  // low-level perturbed steps
    step_t pert_with_list;	  // low-level *interparticle force*
    				  // calculations using perturber list
    step_t pert_without_list;	  // low-level *force* calculations
				  // without perturber list
    step_t perturber_overflow;	  // perturber list overflows

    step_t full_unpert_step;	  // full unperturbed steps
    step_t full_unpert_orbit;	  // full unperturbed orbits
    step_t partial_unpert_orbit;  // partial unperturbed orbits

    step_t tree_change;		  // changes in tree structure

    step_t top_level_combine;	  // top-level combines within tree structure
    step_t low_level_combine;	  // low-level combines within tree structure
    step_t top_level_split;	  // top-level splits within tree structure

    step_t leaf_merge;		  // physical mergers between stars

    real initial_etot;		  // total energy at t = 0
    real de_total;		  // total energy change (subdivisions follow)
    real de_merge;		  // energy change due to mergers
    real de_massloss;		  // energy change due to mass loss
    real de_kick;		  // energy change due to supernovae kicks

} kira_stats_t;

#ifndef KIRA_MAIN

// Declarations and definitions local to kira:

// The global counter:

extern kira_stats_t kira_stats;

// Collisions and mergers:

extern real stellar_merger_criterion_sq;
extern real stellar_encounter_criterion_sq;

#else

// Actual declarations (made in kira.C):

kira_stats_t kira_stats;

real stellar_merger_criterion_sq;
real stellar_encounter_criterion_sq;

#endif

// From kira_init:

real total_mass(hdyn * b);
void kira_initialize(int argc, char** argv,
		     hdynptr& b,	// hdyn root node
		     real& delta_t,	// time span of the integration
		     real& dt_log,	// output interval--power of 2
		     real& dt_snap,	// snap output interval
		     real& dt_sstar,	// stellar evolution timestep
		     real& dt_esc,	// escaper removal
		     real& dt_reinit,	// reinitialization interval
		     real& eta,		// time step parameter
		     real& eps,		// softening length
		     real& d_min_sq,	// N^2 * node combine threshhold
		     real& lag_factor,	// node hysteresis factor
		     real& gamma2,	// unperturbed threshhold
		     bool& exact,	// no perturber list if true
		     real& scaled_stripping_radius, // stripping radius
						    // for unit mass
		     bool& save_last_snap,
		     char* snap_save_file,
		     int& n_stop);	// n to terminate simulation

// From kira_runtime:

void clean_up_files();
bool check_file(char* name, bool del = true);
void print_multiples(hdyn * b);
bool kill_multiples(hdyn * b);
void modify_params(hdyn * b, char * name, real& eta, real& lag_factor);
