
       //=======================================================//    _\|/_
      //  __  _____           ___                    ___       //      /|\
     //  /      |      ^     |   \  |         ^     |   \     //          _\|/_
    //   \__    |     / \    |___/  |        / \    |___/    //            /|\
   //       \   |    /___\   |  \   |       /___\   |   \   // _\|/_
  //     ___/   |   /     \  |   \  |____  /     \  |___/  //   /|\     
 //                                                       //            _\|/_
//=======================================================//              /|\

/*
 *  mk_binaries.C: creates a binary part for each binary node
 *.............................................................................
 *    version 1.0:  Feb 1995   S.F. Portegies Zwart
 *.............................................................................
 *  This file includes the following non-local functions:
 *    adddouble - creates a double part for each node
 *.............................................................................
 */
#include "double_star.h"
#include "kepler.h"
#include "dyn.h"

void set_kepler_from_two_nodes(kepler & k, dyn* d1, dyn* d2) {


//	Should be cheked for time!!!!!!!
//	But is present in hdyn
/*  if (d1->get_time() != d2->get_time())
      err_exit("set_kepler_from_two_nodes: inconsistent times");
*/

//    k.set_time(d1->get_time());
//		For the moment set to zero.
    k.set_time(0.0);
    k.set_total_mass( d1->get_mass() + d2->get_mass() );
    k.set_rel_pos( d2->get_pos() - d1->get_pos() );
    k.set_rel_vel( d2->get_vel() - d1->get_vel() );

    k.set_circular_binary_limit(MAX_CIRC_ECC);

    k.initialize_from_pos_and_vel();
}

// Adds a binary to the parent node.
// all units are given in standard stellar evolution units:
// sma in R_\sun.
void adddouble(dyn* parent, real sma, real ecc, real time,
               starbase * primary, 
               starbase * secondary, 
               real mf, real rf, real tf) {

        cerr<<"adddouble:mrt: "<<mf<<" "<<rf<<" "<<tf<<endl;

        starbase * old_starbase = parent->get_starbase();
        base_element * new_double = 
           new double_star(sma, ecc, time, 
                          (base_element*) primary, 
                          (base_element*) secondary, mf, rf, tf);

//		Jes, also a double star has a story.
        new_double->set_star_story(old_starbase->get_star_story());
        parent->set_starbase((starbase *) new_double);
        old_starbase->set_star_story(NULL);// if not, deleting the old starbase
        delete old_starbase;               // would delete its story as well

//              Setup binary from input data.
//		Initialization is performed in the constructor.
        new_double->evolve_element(new_double->get_current_time());
        new_double->dump(cout);
        }

// If Aprpopriate means that  a binary is only added to a 
// two-body system.
// triples and quadruples etc. can be handled by the dinary evolution
// code except for the effect of the supernova.
void add_binary_if_approprate(dyn * parent, real time) {

cerr<<"add_binary_if_approprate(): "<<parent->get_index()<<endl;

/*    if (b->get_parent()->get_oldest_daughter()->get_younger_sister()
              ->get_younger_sister() != NULL)
        err_exit("add_binary_if_apprprate: not a binary node");
*/

    dyn * d = parent->get_oldest_daughter();
    dyn * sister = d->get_elder_sister();
    if (sister == NULL) sister = d->get_younger_sister();
//
    real sma, ecc;
    real mf, rf, tf;

    if (sister->is_leaf() && d->is_leaf()) {
       cout<<time<<": create binary("<<d->get_index()<<", "
                 <<sister->get_index()<<")"<<endl;
       kepler binary;
       mf = d->get_starbase()->conv_m_star_to_dyn(1);
       rf = d->get_starbase()->conv_r_star_to_dyn(1);
       tf = d->get_starbase()->conv_t_star_to_dyn(1);
       set_kepler_from_two_nodes(binary, sister, d);
       sma = d->get_starbase()->conv_r_dyn_to_star(
                binary.get_semi_major_axis()); 
       ecc = binary.get_eccentricity();
cerr<<"a, e: "<<sma<<" "<<ecc<<endl;
       if(ecc>=1)
          cout <<"This is not a binary: e="<<ecc<<endl;
       else
          adddouble(parent, sma, ecc, time, 
                 sister->get_starbase(), d->get_starbase(), 
                 mf, rf, tf);
    }
    else if(d->is_leaf()) {
       cout<<"create multiple ("<<d->get_index()<<", "
           <<sister->get_index()<<")"<<endl;
       cout<<"decent to sister: "<<sister->get_index()<<endl;
       add_binary_if_approprate(sister, time);
    }
    else if(sister->is_leaf()){
       dyn* daughter = d->get_oldest_daughter();

       cout<<"create multiple ("<<d->get_index()<<", "
           <<sister->get_index()<<")"<<endl;
       cout<<"decent to daughter: "<<daughter->get_index()<<endl;
       add_binary_if_approprate(d, time);
    }
    else cout<<"no binary components found"<<endl;
 }


/*-----------------------------------------------------------------------------
 *  mk_binaries  -- for all particles, add a double_star part using
 *                  if apropriate.
 *-----------------------------------------------------------------------------
 */
void  mk_binaries(dyn * b, real T_rel)
    {
cout<<"mk_binaries"<<endl;

       for_all_daughters(dyn, b, bi) {
          if (bi->is_top_level_node())  {
             if (bi->is_leaf())
                cout<<"just a single_star: "<<bi->get_index()<<endl;
             else if(bi->is_low_level_node)
                add_binary_if_approprate(bi, T_rel);
          }
//          else add_binary_if_approprate(bi, T_rel);
       }
    }

#  define  FALSE  0
#  define  TRUE   1

#ifdef TOOLBOX

/*-----------------------------------------------------------------------------
 *  main  --
 *-----------------------------------------------------------------------------
 */
main(int argc, char ** argv)
    {
    int  c;
    bool  a_flag = FALSE;
    bool  e_flag = FALSE;
    bool  t_flag = FALSE;
    bool  M_flag = FALSE;
    bool  R_flag = FALSE;
    bool  T_flag = FALSE;
    bool  p_flag = FALSE;
    bool  s_flag = FALSE;
    bool  c_flag = FALSE;
    real  mf = 1;
    real  rf = 1;
    real  tf = 1;
    real  T_rel = 0;
    real  sma = 100;
    real  ecc = 0;
    stellar_type str_tpe = Main_Sequence;

    char  *comment;
    void addstar(node*, real, stellar_type, real, real, real);
    extern char *poptarg;
    int  pgetopt(int, char **, char *);

    while ((c = pgetopt(argc, argv, "a:e:M:R:T:t:s:c:")) != -1)
	switch(c)
	    {
            case 'a': a_flag = TRUE;
                      sma = atof(poptarg);
                      break;
            case 'e': e_flag = TRUE;
                      ecc = atof(poptarg);
                      break;
            case 'M': M_flag = TRUE;
                      mf = atof(poptarg);
                      break;
            case 'R': R_flag = TRUE;
                      rf = atof(poptarg);
                      break;
            case 'T': T_flag = TRUE;
                      tf = atof(poptarg);
                      break;
	    case 't': T_flag = TRUE;
		      T_rel = atof(poptarg);
		      break;
            case 's': s_flag = TRUE;
                      str_tpe = (stellar_type)atoi(poptarg);
                      break;
	    case 'c': c_flag = TRUE;
		      comment = poptarg;
		      break;
            case '?': cerr <<
		      "usage: addstar [-t #] [-T #] [-c \"..\"]\n";
		      exit(1);
	    }            
    
    dyn *b;

    while (b = get_dyn(cin))
	{
        if (c_flag == TRUE)
            b->log_comment(comment);
        b->log_history(argc, argv);

        addstar(b, T_rel, str_tpe, mf, rf, tf);
        mk_binaries(b, T_rel);

	put_dyn(cout, *b);	
	delete b;
	}
    }

#endif

/* endof: addstar.c */
