//
//  mknodes.C: construct a linked list of unit-mass nodes.
//
//	       Steve McMillan, July 1996
//

#include "node.h"
#include "single_star.h"
#include "main_sequence.h"
//#include "star_state.h"


#ifdef TOOLBOX
local void  evolve_the_stellar_system(node* b, real time) {

      node * bi;

      if (b->get_oldest_daughter() != NULL)
         for (bi=b->get_oldest_daughter(); bi != NULL;
                                           bi=bi->get_younger_sister())
         evolve_the_stellar_system(bi, time);
      else {
         b->get_starbase()->evolve_element(time);
      }
   }

void main(int argc, char ** argv) {

    bool i_flag = false;
    int  n = 1;

    int id=1;
    real mf=0.1,rf=1, tf=1;
    real m_rel=5, m_tot=2, m_core=0.01;
    stellar_type type = Main_Sequence;
    real t_start = 0;
    real t_end   = 100;
    extern char *poptarg;
    int c;
    char* param_string = "in:s:t:T:";

    while ((c = pgetopt(argc, argv, param_string)) != -1)
	switch(c) {

	    case 'i': i_flag = true;
		      break;
	    case 'n': n = atoi(poptarg);
		      break;
            case 't': t_start = atof(poptarg);
                      break;
            case 'T': t_end = atof(poptarg);
                      break;
            case 's': type = (stellar_type)atoi(poptarg);
                      break;
            case '?': params_to_usage(cerr, argv[0], param_string);
		      exit(1);
	}

    if (n <= 0) err_exit("mknodes: N > 0 required!");

    node *root, *by, *bo;
    single_star *sy, *so;

    // Create root node.

    root = new node();
    if (i_flag) root->set_label(0);

    // Create first daughter node.

    bo = new node();
    root->set_oldest_daughter(bo);
    bo->set_parent(root);
    if (i_flag) bo->set_label(1);

    // Create other daughter nodes.

    for (int i = 1; i < n; i++) {
        by = new node();
	if (i_flag) by->set_label(i+1);
	by->set_parent(root);
        bo->set_younger_sister(by);
        by->set_elder_sister(bo);
        bo = by;
    }

    root->log_history(argc, argv);
    addstar(root, t_start, type, mf);
    put_node(cout, *root);
    root->pretty_print_node(cout);
    evolve_the_stellar_system(root, t_end);
cerr<<"relative_mass:" << bo->get_starbase()->get_total_mass()<<endl;
    put_node(cout, *root);
}

#endif

/* end of: mknodes.c */
