//// nstarev: evolve a cluster of single stars.
////          the single stars should be provided in the input stream.
////
//// Options:    -c    comment to put in the starbase log structure.
////             -M    Mass scaling [total cluster mass in solar units].
////             -n    number of output timesteps (timesteps are taken
////                   with constant time intervals) 
////             -R    Dynamical size scaling for the star
////                   [in units of the virial radius].
////             -S    Random seed.
////             -s    Initial stellar type [default is main_sequence].
////             -T    Dynamical time scaling
////                   [in units of the NBODY time].
////             -t    end time of the stellar evolution [in Million year].
////
//++ Notes:
//++  As in starev no kicks are applied in the supernova event.
//++
//++ Example of usage:      
//++  mknode -n 10 | mkmass -u 100 -l 10 | addstar -T 1 | nstarev
//++
//++ See also: addstar
//++           mkmass
//++           mknode
//++           starev


//// Latest version (SPZ:1.0) February 1993.

//   version 1:  Februari 1993   Simon F. Portegies Zwart
//                               spz@grape.c.u-tokyo.ac.jp

//++ Due to overhull currently not operational.

//#include "node.h"
#include "dyn.h"
#include "single_star.h"
#include "main_sequence.h"
//#include "sstar_to_dyn.h"

#ifdef TOOLBOX

#define  SEED_STRING_LENGTH  60

local void  evolve_the_stellar_system(node* b, real time) {

      node * bi;

      if (b->get_oldest_daughter() != NULL)
         for (bi=b->get_oldest_daughter(); bi != NULL;
                                           bi=bi->get_younger_sister())
         evolve_the_stellar_system(bi, time);
      else {
         b->get_starbase()->evolve_element(time);
      }
   }

void main(int argc, char ** argv) {

    bool s_flag = false;
    bool c_flag = false;
    bool m_flag = false;
    int  n = 1;

    int  input_seed, actual_seed;
    int id=1;
    real m_tot=1,r_hm=1, t_hc=1;
    real m_rel=5, m_core=0.01;
    stellar_type type = Main_Sequence;
    real t_start = 0;
    real t_end   = 100;
    char  *comment;
    char  seedlog[SEED_STRING_LENGTH];
    extern char *poptarg;
    int c;
    char* param_string = "M:n:r:s:t:T:";

    check_help();

    while ((c = pgetopt(argc, argv, param_string)) != -1)
	switch(c) {

            case 'r': r_hm = atof(poptarg);
                      break;
            case 'M': m_flag = true;
		      m_tot = atof(poptarg);
                      break;
	    case 'n': n = atoi(poptarg);
		      break;
            case 'T': t_hc = atof(poptarg);
                      break;
            case 't': t_end = atof(poptarg);
                      break;
            case 'S': type = (stellar_type)atoi(poptarg);
                      break;
            case 's': input_seed = atoi(poptarg);
                      break;
            case 'c': c_flag = TRUE;
                      comment = poptarg;
                      break;
            case '?': params_to_usage(cerr, argv[0], param_string);
	    	      get_help();
	    	      exit(1);
	}

    if (s_flag == FALSE)
        input_seed = 0;                         /* default */
    actual_seed = srandinter(input_seed);

    node *b;

    b = get_node(cin);
    if (c_flag == TRUE)
        b->log_comment(comment);
    b->log_history(argc, argv);
    b->get_starbase()->set_stellar_evolution_scaling(m_tot, r_hm, t_hc);

    addstar(b, t_start, type);

    put_node(cout, *b);

    real t_out, dt = t_end/n;
    for (int i=0; i<n; i++) {
      t_out = dt * (i+1);
      
      for_all_daughters(node, b, bi) 
	bi->get_starbase()->evolve_element(t_out);

      put_node(cout, *b);
    }
}

#endif
