//
// neutron_star.C
//
// derived class of class star.
// class neutron_star discribes extended stellar evolution
// for a stellar remnant
// Class neutron_star will automatically create the following
// base classes: star, single star and base_element.
// if linked to STARLAB alse the general stellareveolution base class
// class starbase will be created.
//
// neutron_star can be transformed out of a
// super_giant, wolf_rayet, thorne_zytkow, helium_giant or white_dwarf.
// neutron_star can be transformed into black_hole.

#include "neutron_star.h"
#include "wolf_rayet.h"
#include "super_giant.h"
#include "thorne_zytkow.h"
#include "helium_giant.h"

// This destruction of the own class IS risky but due to the presence of
// root class single_star the address achange of the main_sequence to
// the hertzsprung_gap does not give any problems.
//
// ANSI C++ first creates the base class before the dreived classes are
// created. If this is done in another order we are in deep shit.

     neutron_star::neutron_star(super_giant & g) : single_star(g) {

      delete &g;

      sudden_mass_loss_flag = false;
      real m_tot = get_total_mass();
      core_mass = helium_core_mass();
      envelope_mass = m_tot - core_mass;
      relative_age = 0;

      bool hit_companion = super_nova();
      post_supernova_story();

      evolve_element(current_time);

//		Kick (see ::super_nova()) micht cause coalescence.
      if (hit_companion) 
         direct_hit();
   }

     neutron_star::neutron_star(wolf_rayet & w) : single_star(w) {

      delete &w;

      sudden_mass_loss_flag = false;
      real m_tot = get_total_mass();
      core_mass = helium_core_mass();
      envelope_mass = m_tot - core_mass;
      relative_age = 0;

      bool hit_companion = super_nova();
      post_supernova_story();

      evolve_element_wt(current_time);


//		Output routines called. Just for some specific purpose.
      if (hit_companion) 
         direct_hit();
   }

     neutron_star::neutron_star(thorne_zytkow & t) : single_star(t) {

      delete &t;

//		TZO core is the neutron-star itself!
      sudden_mass_loss_flag = false;
      relative_age = 0;

      if (envelope_mass>0) {
         if(is_binary_component()) get_binary()->adjust_binary_after_wind_loss(
                        this, envelope_mass, POST_AGB_TIME);
         else reduce_donor_mass(envelope_mass);
      }

      evolve_element(current_time);


//		Thorne-Zytkov stars are generally single and will not
//		coalide with any companion.
   }

     neutron_star::neutron_star(helium_giant & h) : single_star(h) {

      delete &h;

      sudden_mass_loss_flag = false;
      real m_tot = get_total_mass();
      core_mass = helium_core_mass();
      envelope_mass = m_tot - core_mass;
      relative_age = 0;

      bool hit_companion = super_nova();

//		Evolve the newly created neutron-star without 
//		possible transformation.
//		Enables stability of the solution.
      evolve_element_wt(current_time);
      post_supernova_story();

//		Coalescence allowed.
      if (hit_companion) 
         direct_hit();
   }

     neutron_star::neutron_star(white_dwarf & w) : single_star(w) {

      delete &w;

      sudden_mass_loss_flag = false;
      real m_tot = get_total_mass();
      core_mass = cnsts.mass.neutron_star - aic_binding_energy();
      envelope_mass = m_tot - core_mass;
      relative_age = 0;

      bool hit_companion = super_nova();
      post_supernova_story();

      evolve_element(current_time);

      if (hit_companion) 
         direct_hit();
   }

	// ...
 		// Destructors.
 	// ...
// Initial neutron_star should be properly initialized.
void neutron_star::adjust_initial_star() {

//		Keep relative masses constant as from imput.
//      core_mass = get_total_mass();
//      envelope_mass = 0;
      relative_age = max(current_time, 0.0);
      effective_radius = core_radius = radius = cnsts.radius.neutron_star;

      if (relative_mass<cnsts.mass.main_sequence2neutron_star) {
         cerr<<"neutron_star with low mass!"
             <<"\n should be turned into a White_Dwarf"
             <<"\nadopt minimum NS relative mass"<<endl;
         relative_mass = cnsts.mass.main_sequence2neutron_star;
         return;
      }
   }

		// Member function definition.
// evolve a neutron star upto time argument according to
// the model discribed by Eggleton et al. 1989.
void neutron_star::evolve_element(const real end_time) {

//    cerr << "ns evolve element at time " << end_time <<endl;

      real dt = end_time - current_time;
      current_time = end_time;
      relative_age += dt;

//	 	neutron star accretion form disc.
      accrete_from_envelope(dt);

//	collapse to black hole if mass limit exceeded.	
      if (core_mass>cnsts.mass.maximum_neutron_star) {
         core_radius = radius = cnsts.radius.neutron_star; 
         star_transformation_story(Black_Hole);
         new black_hole(*this);
         return;
      }

//		Evolution timescale of neutron star is fixed to
//		Maximal_timestep.
      next_update_age = relative_age + cnsts.safety.maximum_timestep;

//		Also its luminosity is fixed an independed of the
//		accretion. This however can be adapted ealisly.
//    cerr << "ns calc L\n ";
//    PRL(relative_age);
//    dump(cerr);
      luminosity = 40./pow(relative_age + 0.01, 1.4);
      core_radius = radius = cnsts.radius.neutron_star;
      update();
   }

// evolve a neutron star upto time argument according to
// Do, however, not allow transformation into black hole.
// the model discribed by Eggleton et al. 1989.
void neutron_star::evolve_element_wt(const real end_time) {

      real dt = end_time - current_time;
      current_time = end_time;
      relative_age += dt;

      real t_nuc = nucleair_evolution_time();
      next_update_age = relative_age + cnsts.safety.maximum_timestep;

//		Safety for just born neutron-stars.
      if (t_nuc<relative_age)
         luminosity = 40./pow(relative_age - t_nuc, 1.4);
      else {
         luminosity = 100;
      }
      core_radius = radius = cnsts.radius.neutron_star;

      update();
   }

// Detect and opdate spectral features.
// More fun that realy needed.
void neutron_star::update() {

      star::detect_spectral_features();
      last_update_age = relative_age;
      effective_radius = radius;
   }


// Allow neutron star to accrete from a disc.
// Mass accreted by the neutron star is first put into a starage disk
// and on a slower rate accreted onto the neutron star.
// The accretion effeciency NS_CORE_ACCRETION is defined in
// constants.h and has a magnitude of a few percent.
void neutron_star::accrete_from_envelope(const real dt) {

      real mdot = cnsts.accretion.neutron_star
	        * accretion_limit(envelope_mass, dt);

      core_mass += mdot;
      envelope_mass -= mdot;

   }

// Neutron stars do not develope a compact core. They represent 
// their own core.
// Function exist in order to make the compiler happy.
real neutron_star::helium_core_mass() {

//		Fixed neutron-star mass.
      real m = cnsts.mass.neutron_star;

      return m;
   }

// Neutron star's binading energy. Energy fraction is lost during a
// Accretion Induced colaescence of a white dwarf into a neutron star.
// Since whole coalescence is not certain this function only used in
// cases of allowing AIC to occur.
// Typically one of those things that makes programming interesting.
real neutron_star::aic_binding_energy() {

      real GM2_RC2 =  3*cnsts.phys.G*pow(cnsts.sun.mass/cnsts.phys.C, 2)
	           / (5*cnsts.sun.radius);
      real binding_energy = (GM2_RC2*pow(core_mass,2)
			  /  cnsts.radius.neutron_star)/cnsts.sun.mass;

      return binding_energy;
   }

// And there it is: The supernova routine.
// tricky, especially in higher order system.
// One of the few routines that requires the random number generator.
// random numbers:
//	kick_velocity, 
//	theta (kick-angel in orbital plane),
//	phi   (kick-angel perpendicular to orbital plane).
// separation is determined by solving Keplers equation for the 
// Eccentric anomaly taking the mean anomaly randomly.
//
// Return value is boolian whather or not the remnant colescence 
// with its companion.

//supernova routine usin the anomal velocity vector.
bool neutron_star::super_nova() {

//    cerr <<"is_dynamics_involved() "<<is_dynamics_involved()<<endl;;
      sudden_mass_loss_flag = true;

      bool hit_companion = FALSE;
      real v_kick    = random_kick();
      real theta_kick = acos(1-2*random_angle(0, 1));
      real phi_kick   = random_angle(0, 2*PI);
      real x_kick = v_kick*sin(theta_kick)*cos(phi_kick);
      real y_kick = v_kick*sin(theta_kick)*sin(phi_kick);
      real z_kick = v_kick*cos(theta_kick);
      vector kick_velocity(x_kick, y_kick, z_kick);
      anomal_velocity = kick_velocity;
//cerr<< "current anomalous velocity = " << anomal_velocity << " km/s" << endl;

//    if(is_dynamics_involved()) {
//	cerr <<"Supernove: apparently dynamics is involved"<<endl;
	//              If not member of a binary just add kick velocity to
	////              space velocity.
       velocity = sqrt(pow(v_kick, 2) + pow(velocity, 2)
                  + 2*v_kick*velocity*sin(theta_kick)*cos(phi_kick));
         envelope_mass = 0;
         return hit_companion;
   }

//Supernova is not performed by the binary evolution
//but the dynamics performes the adjutments to the evolution of the
//close binary. Note that the same must apply for mass loss in a
//stellar wind and non-conservative mass transfer.
#if 0
     }
      else if(get_binary()->get_bin_type() == Disrupted ||
              get_binary()->get_bin_type() == Merged) {
//cerr <<"Supernove: no dynamics is involved, but binary does not exist"<<endl;
         velocity = sqrt(pow(v_kick, 2) + pow(velocity, 2)
                  + 2*v_kick*velocity*sin(theta_kick)*cos(phi_kick));
         envelope_mass = 0;
         return hit_companion;
      }

      if (get_binary()->get_bin_type() != Merged &&
         get_binary()->get_bin_type() != Disrupted) {
//cerr <<"Supernove: no dynamics is involved"<<endl;
         real a_init = get_binary()->get_semi();
         real e_init = get_binary()->get_eccentricity();
         real mtot_0 = get_binary()->get_total_mass();
         real msn_0 = get_total_mass();
         real m_psn = core_mass;
         real m_comp_0 = mtot_0 - msn_0;
         real m_comp = m_comp_0;

         real separation = random_separation(a_init, e_init);
         real a_new = post_sn_semi_major_axis(a_init, e_init, separation,
                                             msn_0, m_comp_0, m_psn, m_comp,
                                              v_kick, theta_kick, phi_kick);
         real e_new = post_sn_eccentricity(a_init, e_init, separation,
                                           msn_0, m_comp_0, m_psn, m_comp,
                                           v_kick, theta_kick, phi_kick); 
         real v_cm  = post_sn_cm_velocity(a_init, e_init, separation,
                                              msn_0, m_comp_0, m_psn, m_comp,
                                              v_kick, theta_kick, phi_kick);
//              System bound after kick?
         if (e_new>=0 && e_new<1.) {
            get_binary()->set_eccentricity(e_new);
            get_binary()->set_semi(a_new);
//
            get_binary()->set_velocity(v_cm);
            get_binary()->calculate_velocities();
            get_binary()->set_donor_timescale(get_companion());

//              Does it hit the companion?
            real pericenter = a_new*(1-e_new);
            if (pericenter<=get_companion()->get_radius())
               hit_companion = TRUE;
         }
         else {
            spec_type[Runaway]=Runaway;
            get_companion()->set_spec_type(Runaway);
            get_binary()->set_bin_type(Disrupted);
            get_binary()->set_semi(0);

            real e2_init = e_init*e_init;
            real vr_mean_0 = sqrt(((cnsts.phys.G*cnsts.sun.mass
	                   /        cnsts.sun.radius)*(msn_0+m_comp_0)/a_init)
                           *      (1-e2_init)/pow(1+0.5*e2_init, 2));
                 vr_mean_0 /= cnsts.phys.km_per_s;
            real mu_0 = get_total_mass()/mtot_0;
            real v_comp = mu_0*vr_mean_0;

//              Now correct for the velocity at infinity.
            real v_sn_0 = (1-mu_0)*vr_mean_0;
            real v_sn   = sqrt(v_sn_0*v_sn_0 + v_kick*v_kick
                        + 2*v_sn_0*v_kick*sin(theta_kick)*cos(phi_kick));

//              Now corect for binary CM velocity:
            real v_cm = get_binary()->get_velocity();
            v_comp = sqrt(pow(v_comp, 2) + pow(v_cm, 2)
                   + 2*v_comp*v_cm*cos(theta_kick));
            get_companion()->set_velocity(v_comp);
            v_sn = sqrt(pow(v_sn, 2) + pow(v_cm, 2)
                 + 2*v_sn*v_cm*cos(theta_kick));
            velocity = v_sn;
         }
      }
      envelope_mass = 0;

//              Return wheater or not companion was hit.
      return hit_companion;
   }
#endif


// Reduce the mass of the donor when it's filling its Roche-lobe.
// For a neutron star this is, of cource, not appropriate.
star* neutron_star::subtrac_mass_from_donor(const real dt, real& mdot) {

      mdot = mass_ratio_mdot_limit(
                  accretion_limit(envelope_mass, dt));

//		The disc might lose some mass.
      envelope_mass -= mdot;

      return this;
     }

// Force accrete onto neutron star.
real neutron_star::add_mass_to_accretor(const real mdot) {

      relative_mass = max(relative_mass, get_total_mass() + mdot);
      envelope_mass += mdot;

      return mdot;
   }

// Accrete ont neutron star.
// Neutron star accretes with Eddington limit. The accreted mass first
// enters a starage envelope (disc) and is finally accreted onte 
// the compact object.
real neutron_star::add_mass_to_accretor(real mdot, const real dt) {

      mdot = accretion_limit(mdot, dt);
      relative_mass = max(relative_mass, get_total_mass() + mdot);
      envelope_mass += mdot;

      return mdot;
   }

// Limit the accretion onto the nuetron star by the Eddingtopn limit.
real neutron_star::accretion_limit(const real mdot, const real dt) {

      real eddington = 1.5e-08*cnsts.sun.radius*radius*dt;

      return min(mdot, eddington);
   }

// Called from spiral in an common envelope.
// Merge star with primary neutron star.
void neutron_star::merge_elements(star* str) {

      envelope_mass = 0;
      real merger_core = str->get_core_mass();

//      add_mass_to_accretor(str->get_envelope_mass(), SPI_TIME);
      add_mass_to_accretor(str->get_envelope_mass());
      relative_mass = max(relative_mass, get_total_mass() + merger_core);
      core_mass += merger_core;

      spec_type[Merger]=Merger;
      evolve_element_wt(current_time);
   }

// Could happen due to reradiation of the core.
// Generally neutron stars do not supply mass from either wind or 
// Roche-lobe overflow.
star* neutron_star::reduce_donor_mass(const real mdot) {

      if (mdot>envelope_mass) 
         envelope_mass = 0;
      else envelope_mass -= mdot;
      return this;
   }

// Colide and merge just created neutron star with its companion.
// Call is rare but could happen is high velocity kicks are included.
void neutron_star::direct_hit() {

      real theta = random_angle(0, 2*PI);
      real v_bin = get_binary()->get_velocity();
      real ek_ns = get_total_mass()*velocity*velocity;
      real ek_comp = get_companion()->get_total_mass()
                   * pow(get_companion()->get_velocity(), 2);
      real v_merger = sqrt((ek_ns+ek_comp)/get_binary()->get_total_mass());
      real v_new = sqrt(pow(v_merger, 2) + pow(v_bin, 2)
                 + 2*v_merger*v_bin*cos(theta));

      get_binary()->set_semi(0);
      if (get_total_mass() >= get_companion()->get_total_mass())  
         get_binary()->merge_elements(this, get_companion()); 
      else 
         get_binary()->merge_elements(get_companion(), this); 

      get_binary()->set_bin_type(Merged);
      get_binary()->set_semi(0);
      get_binary()->set_velocity(v_new);
      get_binary()->get_primary()->set_velocity(v_new);

//		Ouput direct hit.
      get_binary()->dump("hit.dat");
   }

// Neutron star's stellar wind only implemented for sake of 
// compilers's happiness.
void neutron_star::stellar_wind(const real dt) {

// 		Complete void function.
// 		Could be put in the header file.
// 		Still here for debugging purposes.
   }

bool neutron_star::sudden_mass_loss() {
//cerr<<"neutron_star::sudden_mass_loss() ";

     if (sudden_mass_loss_flag) {
        sudden_mass_loss_flag=false;
//cerr<< true<<endl;
        return true;
     }

//cerr<< false<<endl;
     return false;

   }
