//
// giant.C
//
// derived class of class star.
// class giant discribes extended stellar evolution
// for a star with shell hydrogen burning.
// Class giant will automatically create the following
// base classes: star, single star and base_element.
// if linked to STARLAB alse the general stellareveolution base class
// class starbase will be created.

#include "giant.h"
#include "sub_giant.h"

		// Constructors.
// ANSI C++ first creates the base class before the dreived classes are
// created.
//
// See for further einformation class star, class main_sequence and
// class hertzsprung_gap.

// Constructor for transforming class sub_giant into class
// giant.
     giant::giant(sub_giant & g) : single_star(g) {

      delete &g;		// delete old sub_giant.

      adjust_next_update_age();
//     	 	Wind constant in constant while processing core helium.
// 		Method is more stable than run-time wind constant determination.
      wind_constant = 5*pow(effective_radius, 2)/get_total_mass();
      if (high_mass_star())
         wind_constant *= 2;

//PRC(wind_constant);PRL(current_time);
      evolve_element(current_time);
   }

	// ...
 		// Destructors.
 	// ...
		// Member function definition.
void giant::adjust_initial_star() {
      real t_ms = main_sequence_time();
      real t_giant = t_ms + hertzsprung_gap_time(t_ms)
                          + base_giant_branch_time(t_ms);
      relative_age = max(t_giant, relative_age);
   }

// evolve a giant star upto time argument according to
// the model discribed by Eggleton et al. 1989.
void giant::evolve_element(const real end_time) {

      real dt = end_time - current_time;
      current_time = end_time;
      relative_age += dt;

//      cerr << "giant::evolve_element entered\n";
      if (relative_age<=next_update_age) {

         luminosity = min(helium_giant_luminosity(),
                          maximum_luminosity());
         radius = (0.25*pow(luminosity, 0.4) 
                +  0.8*pow(luminosity, 0.67))
                / pow(relative_mass, 0.27);
         if (radius>=25) {
            real t_ms = main_sequence_time();
            real t_giant = t_ms + hertzsprung_gap_time(t_ms)
                                + base_giant_branch_time(t_ms);
            real t_he = helium_giant_time(t_ms);

            real ff = abs(relative_age - t_giant)/t_he;
            radius *= pow(25.0/radius, ff);
         }
         else {
//		Stellar size is decreasing!
//              Transform star into horizontal_branch.
//		This assures more stabley mass transfer.
            star_transformation_story(Horizontal_Branch);
//	    cerr << "giant::create hb star\n";
            new horizontal_branch(*this);
            return;
         }
      }
      else {
//		giant lifetime exceeded. 
//		Transform star into super_giant.
         star_transformation_story(Super_Giant);
//	 cerr << "giant::create super giant star\n";
         new super_giant(*this);
         return;
      }
//      cerr << "giant::update\n";

      update();
//      cerr << "giant::stellar_wind\n";
      stellar_wind(dt);
//      cerr << "giant::return\n";
   }

// Mass transfer utilities.
// See main_sequence.C for comments.
star* giant::reduce_donor_mass(const real mdot) {

      if (envelope_mass<=mdot) {
         envelope_mass = 0;
         star_transformation_story(Helium_Star);
         return dynamic_cast(star*, new helium_star(*this));
      }

      envelope_mass -= mdot;
      return this;
   }

star* giant::subtrac_mass_from_donor(const real dt, real& mdot ) {

      real mdot_temp = relative_mass*dt/get_binary()->get_donor_timescale();
      mdot = mass_ratio_mdot_limit(mdot_temp);

      if (envelope_mass<mdot) {
         mdot = envelope_mass;
         envelope_mass = 0;
         star_transformation_story(Helium_Star);
         return dynamic_cast(star*, new helium_star(*this));
      }

      envelope_mass -= mdot;
      return this;
   }

//      relative age adjusted lineairly with accreted mass.
void giant::adjust_accretor_age(const real mdot, const bool rejuvenate=true) {

      real m_tot_new = get_total_mass() + mdot;
      real m_rel_new = max(m_tot_new, relative_mass);

      real t_ms_old = main_sequence_time();
      real t_giant_old = t_ms_old + hertzsprung_gap_time(t_ms_old) 
                                  + base_giant_branch_time(t_ms_old);
      real t_he_old = helium_giant_time(t_ms_old);

      real t_ms = main_sequence_time(m_rel_new);
      real t_giant_new = t_ms + hertzsprung_gap_time(m_rel_new, t_ms) 
                              + base_giant_branch_time(m_rel_new, t_ms);
      real t_he_new = helium_giant_time(t_ms);
      real dtime = relative_age - t_giant_old;

      relative_age = t_giant_new
                   + dtime*(t_he_new/t_he_old)*(1-mdot/m_tot_new);

      next_update_age = t_giant_new+t_he_new;
   }

void giant::adjust_next_update_age() {

      real t_ms = main_sequence_time();
      real t_giant = t_ms + hertzsprung_gap_time(t_ms)
                          + base_giant_branch_time(t_ms);
      real t_he = helium_giant_time(t_ms);
      next_update_age = t_giant + t_he;
//cerr<<"giant::adjust_next_update_age ";
//PRC(next_update_age);PRL(current_time);
   }

real giant::stellar_radius(const real mass, const real age) {

      real t_ms = main_sequence_time(mass);
      real t_giant = t_ms + hertzsprung_gap_time(mass, t_ms)
                          + base_giant_branch_time(mass, t_ms);
      real t_he = helium_giant_time(mass, t_ms);

      real l_gs = min(helium_giant_luminosity(mass),
                      maximum_luminosity(mass));
      real r_gs = (0.25*pow(l_gs, 0.4)
                +  0.8*pow(l_gs, 0.67))
                / pow(mass, 0.27);
      if (r_gs>=25) {
         real ff = abs(age - t_giant)/t_he;
         r_gs *= pow(25.0/r_gs, ff);
      }

      return r_gs;
   }

