//
// sub_giant.C
//

#include "sub_giant.h"
#include "hertzsprung_gap.h"

// ANSI C++ first creates the base class before the dreived classes are
// created. 

     sub_giant::sub_giant(hertzsprung_gap & h) : single_star(h) {

       delete &h; 

       adjust_next_update_age();

       instantaneous_element();
       update();

       post_constructor();
}


#if 0
void sub_giant::adjust_initial_star() {

  if(relative_age<=0) {
    real t_ms = main_sequence_time();
    relative_age = max(t_ms + hertzsprung_gap_time(t_ms), relative_age);
  }
}
#endif

void sub_giant::instantaneous_element() {

  real l_g = giant_luminosity();
  real t_ms = main_sequence_time();
  real tend_hg = t_ms + hertzsprung_gap_time(t_ms);
  real t_gs = 0.15*main_sequence_time();

  luminosity = l_g*pow(t_gs/tend_hg, 1.17);
  luminosity = min(luminosity, maximum_luminosity());

  effective_radius = 
            radius = (0.25*pow(luminosity, 0.4) + 0.8*pow(luminosity, 0.67))
                   / pow(relative_mass, 0.27);
}

// evolve a sub_giant star upto time argument according to
// the model discribed by Eggleton et al. 1989.
void sub_giant::evolve_element(const real end_time) {

      real dt = end_time - current_time;
      current_time = end_time;
      relative_age += dt;

      if (relative_age<=next_update_age) {
         real l_g = giant_luminosity();
         real t_ms = main_sequence_time();
         real tend_hg = t_ms + hertzsprung_gap_time(t_ms);
         real t_gs = 0.15*main_sequence_time();

	 real tau = t_gs / (tend_hg + t_gs - relative_age);
	 real tau_prev = t_gs
	              / (tend_hg + t_gs - (relative_age-dt));
	 
         luminosity = l_g*pow(tau, 1.17);
         luminosity = min(luminosity, maximum_luminosity());

         radius = (0.25*pow(luminosity, 0.4) + 0.8*pow(luminosity, 0.67))
                / pow(relative_mass, 0.27);

	 // Not a miss C++ contest winner but it might works.
	 // (SPZ+GN:23 Sep 1998)
	 real m_tot = get_total_mass();
	 real new_mcore;
         if (relative_mass <=
	     cnsts.parameters(upper_ZAMS_mass_for_degenerate_core)) {
	   new_mcore = max(core_mass, 0.146*pow(luminosity, 0.143));
	 }
	 else {
	   new_mcore = core_mass 
	                + l_g*6*t_gs*(pow(tau,1./6)-pow(tau_prev,1./6.))
	                / cnsts.parameters(energy_to_mass_in_internal_units);
	 }
	 
	 core_mass = min(m_tot, new_mcore);
	 envelope_mass = m_tot - core_mass;

				    
      }
      else {
//		sub_giant lifetime exceeded. Transform star into
//		horizontal branch star.
         star_transformation_story(Horizontal_Branch);
         new horizontal_branch(*this);
         return;
      }

      update();
      stellar_wind(dt);
   }

star* sub_giant::reduce_mass(const real mdot) {

      if (envelope_mass<=mdot) {
         envelope_mass = 0;
         star_transformation_story(Helium_Star);
         return dynamic_cast(star*, new helium_star(*this));
      }

      envelope_mass -= mdot;
      return this;
   }

star* sub_giant::subtrac_mass_from_donor(const real dt, real& mdot) {

      mdot = relative_mass*dt/get_binary()->get_donor_timescale();
      mdot = mass_ratio_mdot_limit(mdot);

      if (envelope_mass<=mdot) {
         mdot = envelope_mass;
         envelope_mass = 0;
         star_transformation_story(Helium_Star);
         return dynamic_cast(star*, new helium_star(*this));
      }

      envelope_mass -= mdot;
      return this;
   }

void sub_giant::adjust_accretor_age(const real mdot,
				    const bool rejuvenate=true) {

     real t_ms = main_sequence_time();
     real tend_hg_old = hertzsprung_gap_time(t_ms) + t_ms;
     real t_bgb_old = base_giant_branch_time(t_ms);

     real m_tot_new = get_total_mass() + mdot;
     real m_rel_new = max(m_tot_new, relative_mass);

     t_ms = main_sequence_time(m_rel_new);
     real tend_hg_new = t_ms + hertzsprung_gap_time(m_rel_new, t_ms);
     real t_bgb_new = base_giant_branch_time(m_rel_new, t_ms);

     real dtime = relative_age - tend_hg_old;

     // For relative_mass > 20.97184... (see hertzsprung_gap.C)
     // sub_giants can not exist. (SPZ+GN:10 Oct 1998)
     
     if (t_bgb_new>0) {
       relative_age = tend_hg_new 
                    + dtime*(t_bgb_new/t_bgb_old);
       if (rejuvenate)
          relative_age *= rejuvenation_fraction(mdot/m_tot_new);

       relative_age = max(relative_age, tend_hg_new);
       
     }
     else
        relative_age = next_update_age;

     // next_update_age should not be reset here
     // next_update_age = tend_hg_new+t_bgb_new;

   }

void sub_giant::adjust_next_update_age() {

      real t_ms = main_sequence_time();
      real t_giant = t_ms + hertzsprung_gap_time(t_ms)
                          + base_giant_branch_time(t_ms);
      
      next_update_age = t_giant;
   }

void sub_giant::detect_spectral_features() {

      single_star::detect_spectral_features();


      if (accreted_mass>=cnsts.parameters(B_emission_star_mass_limit))
	spec_type[Emission]=Emission;

}

#if 0
real sub_giant::stellar_radius(const real mass, const real new_age) {

      real t_ms = main_sequence_time(mass);
      real t_giant = t_ms + hertzsprung_gap_time(mass, t_ms)
                          + base_giant_branch_time(mass, t_ms);

      real age = new_age;
      if(new_age<t_giant)
        age = new_age;

      real l_g = giant_luminosity(mass);
      real tend_hg = t_ms + hertzsprung_gap_time(mass, t_ms);
      real t_gs = 0.15*t_ms;

      real l_sg = l_g*pow(t_gs/(tend_hg + t_gs - age), 1.17);
           l_sg = min(l_sg, maximum_luminosity(mass));

      real r_sg = (0.25*pow(l_sg, 0.4) + 0.8*pow(l_sg, 0.67))
                / pow(mass, 0.27);

      return r_sg;
   }
#endif

real sub_giant::gyration_radius_sq() {

  return cnsts.parameters(convective_star_gyration_radius_sq); 
}



